# coding=utf-8
from __future__ import print_function, absolute_import, unicode_literals

# 基本api
import getopt
import sys

from nf.model.storage import context

from nf import __version__
from nf.enum import *

from nf.csdk.c_sdk import py_nfi_set_version
from .basic import (
    set_token, get_version, subscribe, unsubscribe,
    current, get_strerror, schedule, run, log, set_serv_addr, stop, now)

# 交易api
from .trade import (
    order_volume, order_value, order_batch, order_cancel, order_cancel_all, get_positions,
    get_orders, get_unfinished_orders, get_execution_reports,
    get_symbols_by_front, get_symbols_by_fc, get_symbol_of_coins,
    get_all_symbols_of_coin, get_symbol, get_execution_reports_intraday,
    get_orders_intraday, get_instruments,
    history, history_n, get_unfinished_orders_intraday,
    get_exchange_rate,
    get_depth, alert_msg,
    order_future, get_symbol_for_future, get_future_instruments, get_future_instruments_by_period,
    get_future_instruments_by_base, get_stats, get_mark_price, get_settlements, get_funding_rate,
    get_margin_assets, set_mode, set_leverage, add_margin)

# 数据查询api
from .query import (get_trades)

from .strategies import run_strategy_duidao, run_strategy_budan, update_strategy_datas, add_strategy_rds,\
    run_strategy_kongpan
__all__ = [
    'context',
    'set_token', 'get_version', 'subscribe', 'unsubscribe',
    'current', 'get_strerror', 'schedule', 'run', 'get_instruments',
    'log', 'stop', 'now', 'set_serv_addr',

    'order_volume', 'order_value', 'order_batch', 'order_cancel', 'order_cancel_all',
    'get_positions', 'get_orders', 'get_unfinished_orders', 'get_execution_reports',
    'get_execution_reports_intraday', 'get_orders_intraday',
    'get_unfinished_orders_intraday',

    'get_symbols_by_front', 'get_symbols_by_fc', 'get_symbol_of_coins',
    'get_all_symbols_of_coin', 'get_symbol',

    'history', 'history_n',

    'get_exchange_rate',
    'get_depth',
    'alert_msg',

    'get_trades',

    'order_future', 'get_symbol_for_future', 'get_future_instruments', 'get_future_instruments_by_period',
    'get_future_instruments_by_base', 'get_stats', 'get_mark_price', 'get_settlements', 'get_funding_rate',
    'get_margin_assets', 'set_mode', 'set_leverage', 'add_margin',

    'run_strategy_duidao', 'run_strategy_budan', 'update_strategy_datas', 'add_strategy_rds','run_strategy_kongpan',

    'SecurityType_Physical',
    'SecurityType_Future',
    'ExecType_Unknown',
    'ExecType_Failed',
    'ExecType_New',
    'ExecType_PartiallyFilled',
    'ExecType_PartiallyCanceled',
    'ExecType_Filled',
    'ExecType_Canceled',
    'ExecType_Expired',
    'ExecType_Rejected',
    'ExecType_PendingNew',
    'ExecType_CancelRejected',

    'OrderStatus_Unknown',
    'OrderStatus_Failed',
    'OrderStatus_New',
    'OrderStatus_PartiallyFilled',
    'OrderStatus_PartiallyCanceled',
    'OrderStatus_Filled',
    'OrderStatus_Canceled',
    'OrderStatus_Expired',
    'OrderStatus_Rejected',
    'OrderStatus_PendingNew',

    'OrderRejectReason_Unknown',
    'OrderRejectReason_RiskRuleCheckFailed',
    'OrderRejectReason_NoEnoughCash',
    'OrderRejectReason_NoEnoughPosition',
    'OrderRejectReason_IllegalAccountId',
    'OrderRejectReason_IllegalStrategyId',
    'OrderRejectReason_IllegalSymbol',
    'OrderRejectReason_IllegalVolume',
    'OrderRejectReason_IllegalPrice',
    'OrderRejectReason_AccountDisabled',
    'OrderRejectReason_AccountDisconnected',
    'OrderRejectReason_AccountLoggedout',
    'OrderRejectReason_NotInTradingSession',
    'OrderRejectReason_OrderTypeNotSupported',
    'OrderRejectReason_Throttle',
    'OrderRejectReason_SymbolSuspended',
    'OrderRejectReason_Internal',
    'CancelOrderRejectReason_OrderFinalized',
    'CancelOrderRejectReason_UnknownOrder',
    'CancelOrderRejectReason_BrokerOption',
    'CancelOrderRejectReason_AlreadyInPendingCancel',

    'OrderSide_Unknown',
    'OrderSide_Buy',
    'OrderSide_Sell',
    'OrderSide_Buy_Open',
    'OrderSide_Buy_Close',
    'OrderSide_Sell_Open',
    'OrderSide_Sell_Close',
    'OrderSide_Sell_Liquidation',
    'OrderSide_Sell_Reduce',
    'OrderSide_Buy_Liquidation',
    'OrderSide_Buy_Reduce',

    'OrderType_Unknown',
    'OrderType_Limit',
    'OrderType_Market',
    'OrderType_StopMarket',
    'OrderType_StopLimit',
    'OrderType_ProfitMarket',
    'OrderType_ProfitLimit',

    'OrderDuration_Unknown',
    'OrderDuration_FAK',
    'OrderDuration_FOK',
    'OrderDuration_GFD',
    'OrderDuration_GFS',
    'OrderDuration_GTD',
    'OrderDuration_GTC',
    'OrderDuration_GFA',

    'OrderQualifier_Unknown',
    'OrderQualifier_BOC',
    'OrderQualifier_BOP',
    'OrderQualifier_B5TC',
    'OrderQualifier_B5TL',

    'OrderStyle_Unknown',
    'OrderStyle_Volume',
    'OrderStyle_Value',

    'PositionSide_Unknown',
    'PositionSide_Long',
    'PositionSide_Short',

    'PositionEffect_Unknown',
    'PositionEffect_Open',
    'PositionEffect_Close',
    'PositionEffect_CloseToday',
    'PositionEffect_CloseYesterday',

    'CashPositionChangeReason_Unknown',
    'CashPositionChangeReason_Trade',
    'CashPositionChangeReason_Inout',

    'MODE_LIVE',
    'MODE_BACKTEST',
    'ADJUST_NONE',
    'ADJUST_PREV',
    'ADJUST_POST',

    'ExecInstType_PostOnly',
    'ExecInstType_MarkPrice',
    'ExecInstType_IndexPrice',
    'ExecInstType_LastPrice',
    'ExecInstType_ReduceOnly',
]

try:
    __all__ = [str(item) for item in __all__]
    py_nfi_set_version(__version__.__version__, 'python')

    command_argv = sys.argv[1:]
    options, args = getopt.getopt(command_argv, None,
                                  ['strategy_id=', 'filename=',
                                   'mode=', 'token=',
                                   'backtest_start_time=',
                                   'backtest_end_time=',
                                   'backtest_initial_cash=',
                                   'backtest_transaction_ratio=',
                                   'backtest_commission_ratio=',
                                   'backtest_slippage_ratio=',
                                   'backtest_adjust=',
                                   'backtest_check_cache=',
                                   'serv_addr='
                                   ])

    for option, value in options:
        if option == '--serv_addr' and value:
            set_serv_addr(value)
            break
except BaseException as e:
    pass



# coding=utf-8
from __future__ import unicode_literals, print_function, absolute_import
import time
import six
from typing import Dict, Text, Any, List, Union
from datetime import date, datetime, timedelta
from google.protobuf.message import Message
from google.protobuf.timestamp_pb2 import Timestamp
from nf.constant import DATA_TYPE_TICK

GmDate = Union[Text, datetime, date]  # 自定义gm里可表示时间的类型
TextNone = Union[Text, None]  # 可表示str或者None类型


def str_lowerstrip(s):
    # type: (Text) -> Text
    """把字符串进行lower跟strip操作"""
    return s.lower().strip() if s else ''


def date2datetime(d):
    # type: (Union[date,str]) -> datetime
    """
    把date类型转换为datetime类型
    """
    if isinstance(d, six.string_types):
        if len(d) == 8:
            return datetime.strptime(d, '%Y%m%d')
        if len(d) == 10:
            return datetime.strptime(d, '%Y-%m-%d')
        raise Exception('字符串{}不能转为datetime'.format(d))
    return datetime.combine(d, datetime.min.time())


def protobuf_timestamp2datetime(timestamp):
    # type: (Timestamp) -> Union[datetime, None]
    """
    把 protobuf 的 timestamp 转为 datetime类型
    :return:
    """
    if timestamp is None:
        return None
    deltasec = timestamp.seconds + timestamp.nanos / float(_NANOS_PER_SECOND)
    return datetime(1970, 1, 1) + timedelta(seconds=deltasec)


# 来自于 google.protobuf.internal.well_nkown_types.py 里定义的变量
_NANOS_PER_SECOND = 1000000000

def protomessage2dict(protomessageobj, dictobj, *keys):
    # type: (Message, Dict[Text, Any], List[Text]) -> None
    """
    把 proto的message 上的指定的property,附值到dictobj上.其中timestamp会转换为 datetime类型
    """
    for k in keys:
        pv = getattr(protomessageobj, k, None)
        if pv is not None:
            if isinstance(pv, Timestamp):
                if pv.ListFields():
                    # pv.ToDatetime()  不要使用这个方法, 这个方法在python32位时, 最小只能转到 1969-12-31 12:00:00, 因为受限于
                    # ValueError: timestamp out of range for platform localtime()/gmtime() function
                    deltasec = pv.seconds + pv.nanos / float(_NANOS_PER_SECOND)
                    dictobj[k] = datetime(1970, 1, 1) + timedelta(seconds=deltasec)
                else:
                    dictobj[k] = None
            else:
                dictobj[k] = pv


def to_datestr(d):
    # type: (GmDate) -> TextNone
    """
    把datetime.date或datetime.datetime, 或者 yyyy-mm-dd, yyyymmdd 表示日期的字符串统一转换为 yyyy-mm-dd的字符串
    如果不能转换返回None
    """
    date_str = ''
    if isinstance(d, (date, datetime)):
        date_str = d.strftime('%Y-%m-%d')
    if isinstance(d, six.string_types):
        if len(d) == 8:
            try:
                dt = datetime.strptime(d, '%Y%m%d')
                date_str = dt.strftime('%Y-%m-%d')
            except ValueError as e:
                pass
        if len(d) == 10:
            try:
                datetime.strptime(d, '%Y-%m-%d')
                date_str = d
            except ValueError as e:
                pass

    return date_str if date_str else d


def to_exchange(exchange):
    # type: (Text) -> TextNone
    """转换成正确的交易市场. 如果不存在, 则返回None"""
    exchange_set = {'SHSE', 'SZSE', 'CFFEX', 'SHFE', 'DCE', 'CZCE'}
    if not exchange:
        return None
    s = str(exchange).upper().strip()
    return s if s in exchange_set else None


def load_to_list(value):
    """
    无论输入的是啥类型， 都转成list
    """
    if isinstance(value, list):
        return value

    if isinstance(value, dict):
        return [value]

    if isinstance(value, six.string_types):
        return [item.strip() for item in value.split(',')]
    return value


def load_to_datetime_str(value):
    if isinstance(value, (date, datetime)):
        return value.strftime('%Y-%m-%d %H:%M:%S')

    return value


def load_to_second(value):
    # type: (str) -> int
    value = value.lower()

    if value.endswith('h'):
        return int(value[0:-1]) * 60 * 60

    if value.endswith('d'):
        return int(value[0:-1]) * 60 * 60 * 24

    if value.endswith('s'):
        return int(value[0:-1])

    raise ValueError('仅支持s(秒), h(小时), d(天) 结尾')


class DictLikeObject(dict):
    """A dict that allows for object-like property access syntax."""
    def __getattr__(self, name):
        try:
            return self[name]
        except KeyError:
            raise AttributeError(name)

    def __setattr__(self, item, value):
        self[item] = value

    def __str__(self):
        return super(DictLikeObject, self).__str__()

    def __repr__(self):
        return super(DictLikeObject, self).__repr__()


class ObjectLikeDict(object):
    def __init__(self, data):
        self.data = data

    def __getitem__(self, item):
        return self.data.get(item)

    # 如果前面没有定义过的属性， 就会调用这个函数
    def __getattr__(self, item):
        return self.data.get(item)

    def __setitem__(self, item, value):
        self.data[item] = value

    def __setattr__(self, item, value):
        if item != 'data':
            self.data[item] = value
        else:
            super(ObjectLikeDict, self).__setattr__(item, value)

    def __str__(self):
        return self.data.__str__()

    def __repr__(self):
        return self.data.__repr__()


def timestamp2datetime(stamp):
    if stamp is None:
        return None

    return datetime(1970, 1, 1) + timedelta(seconds=stamp)

# this method doesn't work correctly
def datetime2timestamp(dt, convert_to_utc=False):
    ''' Converts a datetime object to UNIX timestamp in seconds. '''

    if isinstance(dt, datetime):
        if convert_to_utc:  # 是否转化为UTC时间
            dt = dt + timedelta(hours=-8) # 中国默认时区
        timestamp = time.mktime(dt.timetuple())
        return int(timestamp)
    return dt
    '''
    #Note: It is necessary to supply timezone.utc explicitly otherwise.timestamp()
    #assume that your naive datetime object is in local timezone.
    if isinstance(dt, datetime):
        dt = dt.replace(tzinfo=timezone.utc)
        timestamp = dt.timestamp()
        return int(timestamp)
    return dt
    '''



def standard_fields(fields, letter_upper=False):
    if not fields:
        return None, None

    if letter_upper:
        fields_list = [field.strip().upper() for field in fields.split(',')]

    else:
        fields_list = [field.strip().lower() for field in fields.split(',')]

    fields_str = ','.join(fields_list)
    return fields_str, fields_list


def dict_fields_filter(dic, fields):
    if not dict or not fields:
        return {}

    if not isinstance(dic, dict):
        return {}

    if not isinstance(fields, list):
        fields = load_to_list(fields)

    ret_dict = {}
    for field in fields:
        if field in dic:
            ret_dict[field] = dic[field]

    return ret_dict


def takePriceFromDepth(depth):
    return depth['price']

def check_frequency(frequency):
    frequency = frequency.lower()
    if frequency == DATA_TYPE_TICK:
        return frequency
    elif frequency.endswith('s'):
        return frequency
    elif frequency.endswith('m'):
        return '{}s'.format(int(frequency[0:-1]) * 60)
    elif frequency.endswith('h'):
        return '{}s'.format(int(frequency[0:-1]) * 60 * 60)
    elif frequency.endswith('d'):
        return '{}s'.format(int(frequency[0:-1]) * 60 * 60 * 24)
    elif frequency.endswith('w'):
        return '{}s'.format(int(frequency[0:-1]) * 60 * 60 * 24 * 7)
    elif frequency.endswith('y'):
        return '{}s'.format(int(frequency[0:-1]) * 60 * 60 * 24 * 365)
    else:
        return None






# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tradegw/api/tradegw.service.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from core.api import common_pb2 as core_dot_api_dot_common__pb2
from core.api import account_pb2 as core_dot_api_dot_account__pb2
from google.api import annotations_pb2 as google_dot_api_dot_annotations__pb2
from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2
from gogoproto import gogo_pb2 as gogoproto_dot_gogo__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='tradegw/api/tradegw.service.proto',
  package='tradegw.api',
  syntax='proto3',
  serialized_pb=_b('\n!tradegw/api/tradegw.service.proto\x12\x0btradegw.api\x1a\x15\x63ore/api/common.proto\x1a\x16\x63ore/api/account.proto\x1a\x1cgoogle/api/annotations.proto\x1a\x1bgoogle/protobuf/empty.proto\x1a\x14gogoproto/gogo.proto\"Q\n\x18GetAccountConnectionsReq\x12 \n\x06\x66ilter\x18\x01 \x01(\x0b\x32\x10.core.api.Filter\x12\x13\n\x0b\x61\x63\x63ount_ids\x18\x02 \x03(\t\"/\n\x18\x44\x65lAccountConnectionsReq\x12\x13\n\x0b\x61\x63\x63ount_ids\x18\x01 \x03(\t\"G\n\x0eGetAccountsReq\x12 \n\x06\x66ilter\x18\x01 \x01(\x0b\x32\x10.core.api.Filter\x12\x13\n\x0b\x61\x63\x63ount_ids\x18\x02 \x03(\t\"#\n\rDelAccountReq\x12\x12\n\naccount_id\x18\x01 \x01(\t\"N\n\x15GetAccountStatusesReq\x12 \n\x06\x66ilter\x18\x01 \x01(\x0b\x32\x10.core.api.Filter\x12\x13\n\x0b\x61\x63\x63ount_ids\x18\x02 \x03(\t\">\n\x15SetAccountCumInoutReq\x12\x12\n\naccount_id\x18\x01 \x01(\t\x12\x11\n\tcum_inout\x18\x02 \x01(\x01\"h\n\rGetAccountReq\x12 \n\x06\x66ilter\x18\x01 \x01(\x0b\x32\x10.core.api.Filter\x12\x10\n\x08\x65xchange\x18\x03 \x01(\t\x12\x0f\n\x07\x61pi_key\x18\x04 \x01(\t\x12\x12\n\napi_secret\x18\x05 \x01(\t\"\xac\x01\n\x14GetSysMessageListReq\x12 \n\x06\x66ilter\x18\x01 \x01(\x0b\x32\x10.core.api.Filter\x12&\n\x07m_level\x18\x02 \x01(\x0e\x32\x15.core.api.SysMsgLevel\x12$\n\x06m_type\x18\x03 \x01(\x0e\x32\x14.core.api.SysMsgType\x12\x12\n\nstart_time\x18\x04 \x01(\x03\x12\x10\n\x08\x65nd_time\x18\x05 \x01(\x03\"\x87\x01\n\x16GetTradeMessageListReq\x12 \n\x06\x66ilter\x18\x01 \x01(\x0b\x32\x10.core.api.Filter\x12\x10\n\x08\x65xchange\x18\x02 \x01(\t\x12\x13\n\x0bstrategy_id\x18\x03 \x01(\t\x12\x12\n\nstart_time\x18\x04 \x01(\x03\x12\x10\n\x08\x65nd_time\x18\x05 \x01(\x03\"\x89\x01\n\x18GetExecrptMessageListReq\x12 \n\x06\x66ilter\x18\x01 \x01(\x0b\x32\x10.core.api.Filter\x12\x10\n\x08\x65xchange\x18\x02 \x01(\t\x12\x13\n\x0bstrategy_id\x18\x03 \x01(\t\x12\x12\n\nstart_time\x18\x04 \x01(\x03\x12\x10\n\x08\x65nd_time\x18\x05 \x01(\x03\x32\xee\x06\n\x0eTradegwService\x12t\n\x14\x41\x64\x64\x41\x63\x63ountConnection\x12\x1b.core.api.AccountConnection\x1a\x1b.core.api.AccountConnection\"\"\x82\xd3\xe4\x93\x02\x1c\"\x17/v3/account-connections:\x01*\x12}\n\x15GetAccountConnections\x12%.tradegw.api.GetAccountConnectionsReq\x1a\x1c.core.api.AccountConnections\"\x1f\x82\xd3\xe4\x93\x02\x19\x12\x17/v3/account-connections\x12\x81\x01\n\x15\x44\x65lAccountConnections\x12%.tradegw.api.DelAccountConnectionsReq\x1a\x16.google.protobuf.Empty\")\x82\xd3\xe4\x93\x02#\"\x1e/v3/account-connections/delete:\x01*\x12\x80\x01\n\x0fGetOrderSummary\x12%.tradegw.api.DelAccountConnectionsReq\x1a\x19.core.api.OrderSummaryNum\"+\x82\xd3\xe4\x93\x02%\x12#/v3/account-trade/get-order-summary\x12t\n\x13GetTradeMessageList\x12#.tradegw.api.GetTradeMessageListReq\x1a\x1a.core.api.TradeMessageList\"\x1c\x82\xd3\xe4\x93\x02\x16\"\x11/v3/trademsg/list:\x01*\x12|\n\x15GetExecrptMessageList\x12%.tradegw.api.GetExecrptMessageListReq\x1a\x1c.core.api.ExecrptMessageList\"\x1e\x82\xd3\xe4\x93\x02\x18\"\x13/v3/execrptmsg/list:\x01*\x12l\n\x11GetSysMessageList\x12!.tradegw.api.GetSysMessageListReq\x1a\x18.core.api.SysMessageList\"\x1a\x82\xd3\xe4\x93\x02\x14\"\x0f/v3/sysmsg/list:\x01*B0Z.nextfin.com/nfplatform/tradegw/api;tradegw_apib\x06proto3')
  ,
  dependencies=[core_dot_api_dot_common__pb2.DESCRIPTOR,core_dot_api_dot_account__pb2.DESCRIPTOR,google_dot_api_dot_annotations__pb2.DESCRIPTOR,google_dot_protobuf_dot_empty__pb2.DESCRIPTOR,gogoproto_dot_gogo__pb2.DESCRIPTOR,])




_GETACCOUNTCONNECTIONSREQ = _descriptor.Descriptor(
  name='GetAccountConnectionsReq',
  full_name='tradegw.api.GetAccountConnectionsReq',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='filter', full_name='tradegw.api.GetAccountConnectionsReq.filter', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='account_ids', full_name='tradegw.api.GetAccountConnectionsReq.account_ids', index=1,
      number=2, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=178,
  serialized_end=259,
)


_DELACCOUNTCONNECTIONSREQ = _descriptor.Descriptor(
  name='DelAccountConnectionsReq',
  full_name='tradegw.api.DelAccountConnectionsReq',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='account_ids', full_name='tradegw.api.DelAccountConnectionsReq.account_ids', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=261,
  serialized_end=308,
)


_GETACCOUNTSREQ = _descriptor.Descriptor(
  name='GetAccountsReq',
  full_name='tradegw.api.GetAccountsReq',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='filter', full_name='tradegw.api.GetAccountsReq.filter', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='account_ids', full_name='tradegw.api.GetAccountsReq.account_ids', index=1,
      number=2, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=310,
  serialized_end=381,
)


_DELACCOUNTREQ = _descriptor.Descriptor(
  name='DelAccountReq',
  full_name='tradegw.api.DelAccountReq',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='account_id', full_name='tradegw.api.DelAccountReq.account_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=383,
  serialized_end=418,
)


_GETACCOUNTSTATUSESREQ = _descriptor.Descriptor(
  name='GetAccountStatusesReq',
  full_name='tradegw.api.GetAccountStatusesReq',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='filter', full_name='tradegw.api.GetAccountStatusesReq.filter', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='account_ids', full_name='tradegw.api.GetAccountStatusesReq.account_ids', index=1,
      number=2, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=420,
  serialized_end=498,
)


_SETACCOUNTCUMINOUTREQ = _descriptor.Descriptor(
  name='SetAccountCumInoutReq',
  full_name='tradegw.api.SetAccountCumInoutReq',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='account_id', full_name='tradegw.api.SetAccountCumInoutReq.account_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='cum_inout', full_name='tradegw.api.SetAccountCumInoutReq.cum_inout', index=1,
      number=2, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=500,
  serialized_end=562,
)


_GETACCOUNTREQ = _descriptor.Descriptor(
  name='GetAccountReq',
  full_name='tradegw.api.GetAccountReq',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='filter', full_name='tradegw.api.GetAccountReq.filter', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='exchange', full_name='tradegw.api.GetAccountReq.exchange', index=1,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='api_key', full_name='tradegw.api.GetAccountReq.api_key', index=2,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='api_secret', full_name='tradegw.api.GetAccountReq.api_secret', index=3,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=564,
  serialized_end=668,
)


_GETSYSMESSAGELISTREQ = _descriptor.Descriptor(
  name='GetSysMessageListReq',
  full_name='tradegw.api.GetSysMessageListReq',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='filter', full_name='tradegw.api.GetSysMessageListReq.filter', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='m_level', full_name='tradegw.api.GetSysMessageListReq.m_level', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='m_type', full_name='tradegw.api.GetSysMessageListReq.m_type', index=2,
      number=3, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='start_time', full_name='tradegw.api.GetSysMessageListReq.start_time', index=3,
      number=4, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='end_time', full_name='tradegw.api.GetSysMessageListReq.end_time', index=4,
      number=5, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=671,
  serialized_end=843,
)


_GETTRADEMESSAGELISTREQ = _descriptor.Descriptor(
  name='GetTradeMessageListReq',
  full_name='tradegw.api.GetTradeMessageListReq',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='filter', full_name='tradegw.api.GetTradeMessageListReq.filter', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='exchange', full_name='tradegw.api.GetTradeMessageListReq.exchange', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='strategy_id', full_name='tradegw.api.GetTradeMessageListReq.strategy_id', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='start_time', full_name='tradegw.api.GetTradeMessageListReq.start_time', index=3,
      number=4, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='end_time', full_name='tradegw.api.GetTradeMessageListReq.end_time', index=4,
      number=5, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=846,
  serialized_end=981,
)


_GETEXECRPTMESSAGELISTREQ = _descriptor.Descriptor(
  name='GetExecrptMessageListReq',
  full_name='tradegw.api.GetExecrptMessageListReq',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='filter', full_name='tradegw.api.GetExecrptMessageListReq.filter', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='exchange', full_name='tradegw.api.GetExecrptMessageListReq.exchange', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='strategy_id', full_name='tradegw.api.GetExecrptMessageListReq.strategy_id', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='start_time', full_name='tradegw.api.GetExecrptMessageListReq.start_time', index=3,
      number=4, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='end_time', full_name='tradegw.api.GetExecrptMessageListReq.end_time', index=4,
      number=5, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=984,
  serialized_end=1121,
)

_GETACCOUNTCONNECTIONSREQ.fields_by_name['filter'].message_type = core_dot_api_dot_common__pb2._FILTER
_GETACCOUNTSREQ.fields_by_name['filter'].message_type = core_dot_api_dot_common__pb2._FILTER
_GETACCOUNTSTATUSESREQ.fields_by_name['filter'].message_type = core_dot_api_dot_common__pb2._FILTER
_GETACCOUNTREQ.fields_by_name['filter'].message_type = core_dot_api_dot_common__pb2._FILTER
_GETSYSMESSAGELISTREQ.fields_by_name['filter'].message_type = core_dot_api_dot_common__pb2._FILTER
_GETSYSMESSAGELISTREQ.fields_by_name['m_level'].enum_type = core_dot_api_dot_account__pb2._SYSMSGLEVEL
_GETSYSMESSAGELISTREQ.fields_by_name['m_type'].enum_type = core_dot_api_dot_account__pb2._SYSMSGTYPE
_GETTRADEMESSAGELISTREQ.fields_by_name['filter'].message_type = core_dot_api_dot_common__pb2._FILTER
_GETEXECRPTMESSAGELISTREQ.fields_by_name['filter'].message_type = core_dot_api_dot_common__pb2._FILTER
DESCRIPTOR.message_types_by_name['GetAccountConnectionsReq'] = _GETACCOUNTCONNECTIONSREQ
DESCRIPTOR.message_types_by_name['DelAccountConnectionsReq'] = _DELACCOUNTCONNECTIONSREQ
DESCRIPTOR.message_types_by_name['GetAccountsReq'] = _GETACCOUNTSREQ
DESCRIPTOR.message_types_by_name['DelAccountReq'] = _DELACCOUNTREQ
DESCRIPTOR.message_types_by_name['GetAccountStatusesReq'] = _GETACCOUNTSTATUSESREQ
DESCRIPTOR.message_types_by_name['SetAccountCumInoutReq'] = _SETACCOUNTCUMINOUTREQ
DESCRIPTOR.message_types_by_name['GetAccountReq'] = _GETACCOUNTREQ
DESCRIPTOR.message_types_by_name['GetSysMessageListReq'] = _GETSYSMESSAGELISTREQ
DESCRIPTOR.message_types_by_name['GetTradeMessageListReq'] = _GETTRADEMESSAGELISTREQ
DESCRIPTOR.message_types_by_name['GetExecrptMessageListReq'] = _GETEXECRPTMESSAGELISTREQ
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

GetAccountConnectionsReq = _reflection.GeneratedProtocolMessageType('GetAccountConnectionsReq', (_message.Message,), dict(
  DESCRIPTOR = _GETACCOUNTCONNECTIONSREQ,
  __module__ = 'tradegw.api.tradegw.service_pb2'
  # @@protoc_insertion_point(class_scope:tradegw.api.GetAccountConnectionsReq)
  ))
_sym_db.RegisterMessage(GetAccountConnectionsReq)

DelAccountConnectionsReq = _reflection.GeneratedProtocolMessageType('DelAccountConnectionsReq', (_message.Message,), dict(
  DESCRIPTOR = _DELACCOUNTCONNECTIONSREQ,
  __module__ = 'tradegw.api.tradegw.service_pb2'
  # @@protoc_insertion_point(class_scope:tradegw.api.DelAccountConnectionsReq)
  ))
_sym_db.RegisterMessage(DelAccountConnectionsReq)

GetAccountsReq = _reflection.GeneratedProtocolMessageType('GetAccountsReq', (_message.Message,), dict(
  DESCRIPTOR = _GETACCOUNTSREQ,
  __module__ = 'tradegw.api.tradegw.service_pb2'
  # @@protoc_insertion_point(class_scope:tradegw.api.GetAccountsReq)
  ))
_sym_db.RegisterMessage(GetAccountsReq)

DelAccountReq = _reflection.GeneratedProtocolMessageType('DelAccountReq', (_message.Message,), dict(
  DESCRIPTOR = _DELACCOUNTREQ,
  __module__ = 'tradegw.api.tradegw.service_pb2'
  # @@protoc_insertion_point(class_scope:tradegw.api.DelAccountReq)
  ))
_sym_db.RegisterMessage(DelAccountReq)

GetAccountStatusesReq = _reflection.GeneratedProtocolMessageType('GetAccountStatusesReq', (_message.Message,), dict(
  DESCRIPTOR = _GETACCOUNTSTATUSESREQ,
  __module__ = 'tradegw.api.tradegw.service_pb2'
  # @@protoc_insertion_point(class_scope:tradegw.api.GetAccountStatusesReq)
  ))
_sym_db.RegisterMessage(GetAccountStatusesReq)

SetAccountCumInoutReq = _reflection.GeneratedProtocolMessageType('SetAccountCumInoutReq', (_message.Message,), dict(
  DESCRIPTOR = _SETACCOUNTCUMINOUTREQ,
  __module__ = 'tradegw.api.tradegw.service_pb2'
  # @@protoc_insertion_point(class_scope:tradegw.api.SetAccountCumInoutReq)
  ))
_sym_db.RegisterMessage(SetAccountCumInoutReq)

GetAccountReq = _reflection.GeneratedProtocolMessageType('GetAccountReq', (_message.Message,), dict(
  DESCRIPTOR = _GETACCOUNTREQ,
  __module__ = 'tradegw.api.tradegw.service_pb2'
  # @@protoc_insertion_point(class_scope:tradegw.api.GetAccountReq)
  ))
_sym_db.RegisterMessage(GetAccountReq)

GetSysMessageListReq = _reflection.GeneratedProtocolMessageType('GetSysMessageListReq', (_message.Message,), dict(
  DESCRIPTOR = _GETSYSMESSAGELISTREQ,
  __module__ = 'tradegw.api.tradegw.service_pb2'
  # @@protoc_insertion_point(class_scope:tradegw.api.GetSysMessageListReq)
  ))
_sym_db.RegisterMessage(GetSysMessageListReq)

GetTradeMessageListReq = _reflection.GeneratedProtocolMessageType('GetTradeMessageListReq', (_message.Message,), dict(
  DESCRIPTOR = _GETTRADEMESSAGELISTREQ,
  __module__ = 'tradegw.api.tradegw.service_pb2'
  # @@protoc_insertion_point(class_scope:tradegw.api.GetTradeMessageListReq)
  ))
_sym_db.RegisterMessage(GetTradeMessageListReq)

GetExecrptMessageListReq = _reflection.GeneratedProtocolMessageType('GetExecrptMessageListReq', (_message.Message,), dict(
  DESCRIPTOR = _GETEXECRPTMESSAGELISTREQ,
  __module__ = 'tradegw.api.tradegw.service_pb2'
  # @@protoc_insertion_point(class_scope:tradegw.api.GetExecrptMessageListReq)
  ))
_sym_db.RegisterMessage(GetExecrptMessageListReq)


DESCRIPTOR.has_options = True
DESCRIPTOR._options = _descriptor._ParseOptions(descriptor_pb2.FileOptions(), _b('Z.nextfin.com/nfplatform/tradegw/api;tradegw_api'))

_TRADEGWSERVICE = _descriptor.ServiceDescriptor(
  name='TradegwService',
  full_name='tradegw.api.TradegwService',
  file=DESCRIPTOR,
  index=0,
  options=None,
  serialized_start=1124,
  serialized_end=2002,
  methods=[
  _descriptor.MethodDescriptor(
    name='AddAccountConnection',
    full_name='tradegw.api.TradegwService.AddAccountConnection',
    index=0,
    containing_service=None,
    input_type=core_dot_api_dot_account__pb2._ACCOUNTCONNECTION,
    output_type=core_dot_api_dot_account__pb2._ACCOUNTCONNECTION,
    options=_descriptor._ParseOptions(descriptor_pb2.MethodOptions(), _b('\202\323\344\223\002\034\"\027/v3/account-connections:\001*')),
  ),
  _descriptor.MethodDescriptor(
    name='GetAccountConnections',
    full_name='tradegw.api.TradegwService.GetAccountConnections',
    index=1,
    containing_service=None,
    input_type=_GETACCOUNTCONNECTIONSREQ,
    output_type=core_dot_api_dot_account__pb2._ACCOUNTCONNECTIONS,
    options=_descriptor._ParseOptions(descriptor_pb2.MethodOptions(), _b('\202\323\344\223\002\031\022\027/v3/account-connections')),
  ),
  _descriptor.MethodDescriptor(
    name='DelAccountConnections',
    full_name='tradegw.api.TradegwService.DelAccountConnections',
    index=2,
    containing_service=None,
    input_type=_DELACCOUNTCONNECTIONSREQ,
    output_type=google_dot_protobuf_dot_empty__pb2._EMPTY,
    options=_descriptor._ParseOptions(descriptor_pb2.MethodOptions(), _b('\202\323\344\223\002#\"\036/v3/account-connections/delete:\001*')),
  ),
  _descriptor.MethodDescriptor(
    name='GetOrderSummary',
    full_name='tradegw.api.TradegwService.GetOrderSummary',
    index=3,
    containing_service=None,
    input_type=_DELACCOUNTCONNECTIONSREQ,
    output_type=core_dot_api_dot_account__pb2._ORDERSUMMARYNUM,
    options=_descriptor._ParseOptions(descriptor_pb2.MethodOptions(), _b('\202\323\344\223\002%\022#/v3/account-trade/get-order-summary')),
  ),
  _descriptor.MethodDescriptor(
    name='GetTradeMessageList',
    full_name='tradegw.api.TradegwService.GetTradeMessageList',
    index=4,
    containing_service=None,
    input_type=_GETTRADEMESSAGELISTREQ,
    output_type=core_dot_api_dot_account__pb2._TRADEMESSAGELIST,
    options=_descriptor._ParseOptions(descriptor_pb2.MethodOptions(), _b('\202\323\344\223\002\026\"\021/v3/trademsg/list:\001*')),
  ),
  _descriptor.MethodDescriptor(
    name='GetExecrptMessageList',
    full_name='tradegw.api.TradegwService.GetExecrptMessageList',
    index=5,
    containing_service=None,
    input_type=_GETEXECRPTMESSAGELISTREQ,
    output_type=core_dot_api_dot_account__pb2._EXECRPTMESSAGELIST,
    options=_descriptor._ParseOptions(descriptor_pb2.MethodOptions(), _b('\202\323\344\223\002\030\"\023/v3/execrptmsg/list:\001*')),
  ),
  _descriptor.MethodDescriptor(
    name='GetSysMessageList',
    full_name='tradegw.api.TradegwService.GetSysMessageList',
    index=6,
    containing_service=None,
    input_type=_GETSYSMESSAGELISTREQ,
    output_type=core_dot_api_dot_account__pb2._SYSMESSAGELIST,
    options=_descriptor._ParseOptions(descriptor_pb2.MethodOptions(), _b('\202\323\344\223\002\024\"\017/v3/sysmsg/list:\001*')),
  ),
])
_sym_db.RegisterServiceDescriptor(_TRADEGWSERVICE)

DESCRIPTOR.services_by_name['TradegwService'] = _TRADEGWSERVICE

# @@protoc_insertion_point(module_scope)



# coding=utf-8
from __future__ import unicode_literals, print_function, absolute_import

# this is a namespace package
try:
    import pkg_resources
    pkg_resources.declare_namespace(__name__)
except ImportError:
    import pkgutil
    __path__ = pkgutil.extend_path(__path__, __name__)


# coding=utf-8
from __future__ import unicode_literals, print_function, absolute_import

from nf.constant import CSDK_OPERATE_SUCCESS, ERR_INVALID_PARAMETER

from nf.api.basic import _error_call_back, _check_frequency, _reform_tick_data

from nf.csdk.c_sdk import (py_nfi_place_order, py_nfi_get_unfinished_orders,
                           py_nfi_get_orders, py_nfi_cancel_all_orders,
                           py_nfi_get_execution_reports_intraday, py_nfi_get_orders_intraday,
                           py_nfi_get_unfinished_orders_intraday,
                           py_nfi_cancel_order,
                           py_nfi_get_execution_reports, py_nfi_get_positions,
                           py_nfi_get_instruments,
                           py_nfi_get_symbols_by_fc, py_nfi_get_symbols_by_front,
                           py_nfi_get_symbol_of_coins, py_nfi_get_symbol,
                           py_nfi_get_all_symbols_of_coin,
                           py_nfi_history_ticks, py_nfi_history_bars,
                           py_nfi_history_ticks_n, py_nfi_history_bars_n,
                           py_nfi_get_exchange_rate, py_nfi_get_market_depth,
                           nfi_now,
                           py_nfi_alert_msg,
                           py_nfi_get_symbol_for_future, py_nfi_get_target_stats,
                           py_nfi_get_settlements, py_nfi_get_fundingRate,
                           py_nfi_get_margin_assets, py_nfi_set_margin_mode,
                           py_nfi_set_leverage, py_nfi_add_margin,
                           py_nfi_get_mark_price)

from nf.enum import OrderType_Limit, MODE_BACKTEST
from nf.model.storage import Context
from nf.pb.core.api.account_pb2 import (Order, Orders, ExecRpts, Positions, ExchangeRateList, SecurityType_Future,
                                        StatItems, SettlementItems, GetMarginAssetsResp, SecurityType_Physical)
from nf.pb.trade.api.trade.service_pb2 import (GetUnfinishedOrdersReq,
                                               GetOrdersReq, GetExecrptsReq, PlaceOrderReq, GetPositionsReq,
                                               SearchCoinPairsRsp, CancelOrderReq, AlertMeReq, TradingSymbolReq,
                                               SearchCoinPairsReq, TargetSymbolReq, StatsReq, GetSettlementReq,
                                               SymbolReq, FundingRateResp, GetMarginAssetsReq, SetModeReq, StatResp,
                                               SetLeverageReq, SetMarginReq)

from nf.pb.data.api.data_pb2 import Ticks, Bars, DethpRateLists, MarkPrice

from nf.pb_to_dict import protobuf_to_dict, dict_to_protobuf
from nf.utils import load_to_list, dict_fields_filter, takePriceFromDepth, load_to_datetime_str

from nf.pb.data.api.data_pb2 import InstrumentInfos
from nf.pb.core.api.common_pb2 import CommSymbolInfo
from nf.enum import *
import pandas as pd

from datetime import datetime
import time
context = Context()


def _place_order(**kwargs):
    order = Order()
    for key in kwargs:
        setattr(order, key, kwargs[key])
        if context.mode == MODE_BACKTEST:
            order.created_at.seconds = nfi_now()    #get timestamp from csdk

    place_order_req = PlaceOrderReq()

    place_order_req.orders.extend([order])
    req = place_order_req.SerializeToString()
    status, result = py_nfi_place_order(req)

    if not status == CSDK_OPERATE_SUCCESS:

        _error_call_back(status)
        return []

    if not result:
        return []

    res = Orders()
    res.ParseFromString(result)

    return [protobuf_to_dict(res_order, including_default_value_fields=True) for
            res_order in res.data]


def order_volume(symbol, volume, side, order_type, position_effect=0,
                 order_duration=0, order_qualifier=0,
                 account = '', price=0):
    """
    按指定量委托
    """
    if not symbol or not volume or not side or not order_type or len(symbol) <= 0:
        raise TypeError('order_volume() : argument missing or incorrect argument type.')

    if order_type == OrderType_Limit and price <= 0:
        raise TypeError('order_volume() : argument error, whit price <= 0 for order type \'OrderType_Limit\'.')

    symbol_list = symbol.split(".")
    if len(symbol_list) > 1:
        if symbol_list[0] == "BITMEX" or symbol_list[0] == "OKEXFU":
            raise TypeError('Please use order_future to Placeorder!')

    account_id = ''
    account_name = ''

    order_src = 2  # sdk
    return _place_order(symbol=symbol, volume=volume, value=-1,  # when place an order by volume, 'value' should be set to a negtive number
                        side=side, account_id=account_id,
                        account_name=account_name, price=price,
                        order_type=order_type, order_src=order_src,
                        subject=SecurityType_Physical)


def order_value(symbol, value, side, order_type, position_effect=0,
                 order_duration=0, order_qualifier=0,
                 account='', price=0):
    """
    按指定价委托
    """
    if not symbol or not value or not side or not order_type or len(symbol) <= 0:
        raise TypeError('order_value() : argument missing or incorrect argument type.')

    if order_type == OrderType_Limit and price <= 0:
        raise TypeError('order_value() : argument error, whit price <= 0 for order type \'OrderType_Limit\'.')

    symbol_list = symbol.split(".")
    if len(symbol_list) > 1:
        if symbol_list[0] == "BITMEX" or symbol_list[0] == "OKEXFU":
            raise TypeError('Please use order_future to Placeorder!')

    account_id = ''
    account_name = ''

    order_src = 2  # sdk
    return _place_order(symbol=symbol, value=value, volume=-1,  #when place an order by volume, 'value' should be set to a negtive number
                        side=side, account_id=account_id,
                        account_name=account_name, price=price,
                        order_type=order_type, order_src=order_src, subject=SecurityType_Physical)


def order_batch(orders, combine=False, account=''):
    orders = load_to_list(orders)

    place_order_req = PlaceOrderReq()
    try:
        for order in orders:
            pb_order = Order()
            if 'created_at' in order.keys():
                order.pop('created_at')
            if 'updated_at' in order.keys():
                order.pop('updated_at')
            order["order_src"] = 2  # 对于自己组的orders，order_src 必须设置上
            ex = order["symbol"].split(".")
            if len(ex) > 1:
                if ex[0] == "BITMEX" or ex[0] == "OKEXFU":
                    order["subject"] = SecurityType_Future
            pb_order = dict_to_protobuf(pb_klass_or_instance=pb_order, values=order, ignore_none=True)
            place_order_req.orders.extend([pb_order])
    except TypeError:
        raise TypeError('order_batch() : argument missing or incorrect argument type.')
        return []

    req = place_order_req.SerializeToString()
    status, result = py_nfi_place_order(req)

    if not status == CSDK_OPERATE_SUCCESS:

        _error_call_back(status)
        return []

    if not result:
        return []

    res = Orders()
    res.ParseFromString(result)

    return [protobuf_to_dict(res_order, including_default_value_fields=True) for
            res_order in res.data]


def order_future(symbol, volume, side, order_type, leverage=0, price=0,
                 exec_inst='', stop_price=0, account=''):
    if not symbol or not volume or not side or not order_type or len(symbol) <= 0:
        raise TypeError('order_future() : argument missing or incorrect argument type.')

    if volume <= 0:
        raise TypeError('order_future() : volume must > 0')
    if side < OrderSide_Buy_Open or side > OrderSide_Sell_Close:
        raise TypeError('order_future() : side wrong')
    if order_type < OrderType_Limit or order_type > OrderType_ProfitLimit:
        raise TypeError('order_future() : order_type wrong')

    order_src = 2  # sdk

    if isinstance(exec_inst, list):
        for i in range(len(exec_inst)):
            exec_inst[i] = str(exec_inst[i])
        exec_inst = ",".join(exec_inst)

    return _place_order(symbol=symbol, value=-1, volume=volume, side=side, price=price,
                        order_type=order_type, order_src=order_src, leverage=leverage,
                        subject=SecurityType_Future, exec_inst=exec_inst, stop_price=stop_price, account_id=account)


def get_unfinished_orders(exchange=None):
    """
    查询所有未结委托
    """
    req = GetUnfinishedOrdersReq()
    if exchange:
        exchange = exchange.upper()
        req.exchange = exchange

    req = req.SerializeToString()
    status, result = py_nfi_get_unfinished_orders(req)
    if not status == CSDK_OPERATE_SUCCESS:

        _error_call_back(status)
        return []

    res = Orders()
    res.ParseFromString(result)

    orders = [protobuf_to_dict(res_order, including_default_value_fields=True) for res_order in res.data]

    if not exchange:
        return orders
    else:
        orders = [order for order in orders if exchange in order['symbol']]
        return orders


def get_orders(exchange=None, symbols=None, cl_ord_ids=None):
    """
    查询日内全部委托
    """
    req = GetOrdersReq()
    if exchange:
        exchange = exchange.upper()
        req.exchange = exchange

    if symbols:
        symbols = load_to_list(symbols)
        for symbol in symbols:
            symbol = str(symbol).split(".")  # 只能要 'BINANCE.BTCUSDT' 中的 BTCUSDT
            if len(symbol) > 1:
                symbol = symbol[1]
            req.symbols.append(symbol)

    if cl_ord_ids:
        cl_ord_ids = load_to_list(cl_ord_ids)
        for cl_ord_id in cl_ord_ids:
            req.cl_ord_ids.append(cl_ord_id)

    req = req.SerializeToString()
    status, result = py_nfi_get_orders(req)

    if not status == CSDK_OPERATE_SUCCESS:
        _error_call_back(status)
        return []

    if not result:
        return []

    res = Orders()
    res.ParseFromString(result)

    orders = [protobuf_to_dict(res_order, including_default_value_fields=True) for res_order in res.data]

    orders.sort(key=_created_time, reverse=True)

    if not exchange:
        return orders
    else:
        orders = [order for order in orders if exchange in order['symbol']]
        return orders


def order_cancel(wait_cancel_orders):
    """
    撤销委托
    """
    wait_cancel_orders = load_to_list(wait_cancel_orders)

    orders = Orders()
    cancel_order_req = CancelOrderReq()

    try:
        cl_ord_ids = [order['cl_ord_id'] for order in wait_cancel_orders]
    except Exception as e:
        #传入的参数不是Order 类
        raise TypeError('order_cancel() : argument missing or incorrect argument type.')
    else:
        for cl_ord_id in cl_ord_ids:
            order = orders.data.add()
            order.cl_ord_id = cl_ord_id
            cancel_order_req.orders.extend([order])

        req = cancel_order_req.SerializeToString()

        status = py_nfi_cancel_order(req)
        if not status == CSDK_OPERATE_SUCCESS:
            _error_call_back(status)


# req = GetExecrptsReq, res = core.api.ExecRpts
def get_execution_reports(exchange=None):
    req = GetExecrptsReq()
    if exchange:
        exchange = exchange.upper()
        req.exchange = exchange

    req = req.SerializeToString()
    status, result = py_nfi_get_execution_reports(req)
    if not status == CSDK_OPERATE_SUCCESS:
        
        _error_call_back(status)
        return []

    res = ExecRpts()
    res.ParseFromString(result)
    reports = [protobuf_to_dict(Execrpt, including_default_value_fields=True) for
            Execrpt in res.data]

    reports.sort(key=_created_time, reverse=True)

    if not exchange:
        return reports
    else:
        reports = [order for order in reports if exchange in order['symbol']]
        return reports


def get_positions(exchange=None, currency=None):
    # 持仓信息
    req = GetPositionsReq()
    if not currency:
        currency = ''

    if not exchange:
        exchange = ''

    exchange = exchange.upper()
    currency = currency.lower()

    req.exchange = exchange
    req.currency = currency

    req = req.SerializeToString()
    status, result = py_nfi_get_positions(req)

    # 如果调用rpc返回的状态不正确
    if not status == CSDK_OPERATE_SUCCESS:

        _error_call_back(status)
        return []

    positions = Positions()
    positions.ParseFromString(result)

    positions = [protobuf_to_dict(position, including_default_value_fields=True)
                 for position in positions.data]

    res = []
    for position in positions:
        t_crc = position["currency"]
        if "." not in t_crc:
            continue
        t_crc_list = t_crc.split(".")
        if('valid_available' in position.keys()):
            del position['valid_available']
        if ('can_reduce' in position.keys()):
            del position['can_reduce']
        if ('close_tfv' in position.keys()):
            del position['close_tfv']
        if ('filled_balance' in position.keys()):
            del position['filled_balance']
        if ('profit' in position.keys()):
            del position['profit']
        if ('pft_close_tfv' in position.keys()):
            del position['pft_close_tfv']
        if ('symbol_old'in position.keys()):
            del position['symbol_old']
        if ("liquidation_price_rl" in position.keys()):
            del position['liquidation_price_rl']

        if len(t_crc_list) != 2:
            continue
        t_exchange = t_crc_list[0]
        t_currency = t_crc_list[1]
        if exchange.strip() and exchange.upper() != t_exchange.upper():
            continue
        if 0 == position['subject']:
            if currency.strip() and currency.upper() != t_currency.upper():
                continue
        if 1 == position['subject']:
            if currency.strip() and currency.upper() != t_crc.upper():
                continue
        '''原position中的currency格式为 交易所代码.币代码，如OKEX.btc，将此字段拆成两个独立的key，即在原posiiton中增加exchange'''
        if 0 == position["subject"]:  # 现货把currency改一下，期货不用改
            position['currency'] = t_currency
        position['exchange'] = t_exchange

        res.append(position)

    return res


def get_symbols_by_front(market, currency, df=False):
    if not market or not currency:
        raise TypeError('get_symbols_by_front() : argument missing or incorrect argument type.')

    market = market.upper()
    currency = currency.lower()

    status, result = py_nfi_get_symbols_by_front(market, currency)

    if not status == CSDK_OPERATE_SUCCESS:
      
        _error_call_back(status)
        return []

    symbols = InstrumentInfos()
    symbols.ParseFromString(result)

    symbols = [protobuf_to_dict(symbol, including_default_value_fields=True) for symbol in symbols.data]

    instrument_infos = [
        '{}'.format(instrumentinfo['symbol']) for instrumentinfo in symbols]

    if not df:
        return instrument_infos

    data = pd.DataFrame(instrument_infos)

    return data


def get_symbols_by_fc(market, from_currency, df=False):
    if not market or not  from_currency:
        raise TypeError('get_symbols_by_fc() : argument missing or incorrect argument type.')

    market = market.upper()
    from_currency = from_currency.lower()

    status, result = py_nfi_get_symbols_by_fc(market, from_currency)

    if not status == CSDK_OPERATE_SUCCESS:
      
        _error_call_back(status)
        return []

    symbols = InstrumentInfos()
    symbols.ParseFromString(result)

    symbols = [protobuf_to_dict(symbol, including_default_value_fields=True)
               for symbol in symbols.data]

    instrument_infos = [
        '{}'.format(symbol['symbol']) for
        symbol in symbols]

    if not df:
        return instrument_infos

    data = pd.DataFrame(instrument_infos)

    return data


def get_symbol_of_coins(market, currency_a, currency_b, df=False):
    if not market or not currency_a or not currency_b:
        raise TypeError('get_symbol_of_coins() : argument missing or incorrect argument type.')

    market = market.upper()

    preq = TradingSymbolReq()
    preq.market = market
  
    preq.coin_a = currency_a.lower()
    preq.coin_b = currency_b.lower()

    req = preq.SerializeToString()

    status, result = py_nfi_get_symbol_of_coins(req)

    if not status == CSDK_OPERATE_SUCCESS:
       
        _error_call_back(status)
        return

    common_symbol_info = CommSymbolInfo()
    common_symbol_info.ParseFromString(result)

    # coins_info = {
    #   '{}'.format(common_symbol_info.data['symbol'])}

    if not df:
        return [common_symbol_info.data.symbole]

    data = pd.DataFrame([common_symbol_info.data.symbole])

    return data


def get_symbol(market, front, rear, df=False):
    if not market or not front or not rear:
        raise TypeError('get_symbol() : argument missing or incorrect argument type.')

    market = market.upper()

    preq = TradingSymbolReq()
    preq.market = market
    
    preq.coin_a = front.lower()
    preq.coin_b = rear.lower()

    req = preq.SerializeToString()

    status, result = py_nfi_get_symbol(req)
    if not status == CSDK_OPERATE_SUCCESS:
       
        _error_call_back(status)
        return None

    common_symbol_info = CommSymbolInfo()
    common_symbol_info.ParseFromString(result)
    '''
    common_symbol_info = [protobuf_to_dict(symbol_info, including_default_value_fields=True)
                          for symbol_info in common_symbol_info.data]
    symbol = {
        '{}'.format(symbol_info['symbol']): symbol_info for
        symbol_info in common_symbol_info}
    '''

    if not df:
        return common_symbol_info.data.symbole

    data = pd.DataFrame([{'symbol': common_symbol_info.data.symbole}])

    return data


def get_execution_reports_intraday(exchange=None):
    if not exchange:
        exchange = ''

    exchange = exchange.upper()

    status, result = py_nfi_get_execution_reports_intraday(exchange)

    if not status == CSDK_OPERATE_SUCCESS:
       
        _error_call_back(status)
        return []

    res = ExecRpts()
    res.ParseFromString(result)

    reports = [protobuf_to_dict(Execrpt, including_default_value_fields=True) for
               Execrpt in res.data]

    reports.sort(key=_created_time, reverse=True)

    if not exchange:
        return reports
    else:
        reports = [order for order in reports if exchange in order['symbol']]
        return reports


def get_orders_intraday(exchange=None):
    if not exchange:
        exchange = ''

    exchange = exchange.upper()

    status, result = py_nfi_get_orders_intraday(exchange)

    if not status == CSDK_OPERATE_SUCCESS:
      
        _error_call_back(status)
        return []

    res = Orders()
    res.ParseFromString(result)

    orders = [protobuf_to_dict(res_order, including_default_value_fields=True) for res_order in res.data]

    orders.sort(key=_created_time, reverse=True)

    if not exchange:
        return orders
    else:
        orders = [order for order in orders if exchange in order['symbol']]
        return orders


def get_all_symbols_of_coin(market, from_currency, df=False):
    if not market or not from_currency:
        raise TypeError('get_all_symbols_of_coin() : argument missing or incorrect argument type.')

    market = market.upper()
    from_currency = from_currency.lower()

    status, result = py_nfi_get_all_symbols_of_coin(market, from_currency)

    if not status == CSDK_OPERATE_SUCCESS:
        
        _error_call_back(status)
        return []

    instruments = InstrumentInfos()
    instruments.ParseFromString(result)

    instruments = [protobuf_to_dict(instrument, including_default_value_fields=True)
                   for instrument in instruments.data]

    coins_info = ['{}'.format(instrument['symbol']) for instrument in instruments]

    if not df:
        return coins_info

    data = pd.DataFrame(coins_info)

    return data


def history(symbol, frequency, start_time, end_time, fields='', skip_suspended=True, fill_missing='', df=False):
    if not symbol or not frequency or not start_time or not end_time:
        raise TypeError('history() : argument missing or incorrect argument type.')

    if len(symbol) <= 0:
        return []

    start_time = load_to_datetime_str(start_time)
    end_time = load_to_datetime_str(end_time)

    if frequency == 'tick':

        status, result = py_nfi_history_ticks(symbol, start_time, end_time, fields, skip_suspended,
                                              fill_missing, 1)

        if not status == CSDK_OPERATE_SUCCESS:

            _error_call_back(status)
            return []

        ticks = Ticks()
        ticks.ParseFromString(result)
        ticks = [protobuf_to_dict(tick) for tick in ticks.data]
        ticks = [_reform_tick_data(tick) for tick in ticks]
        for tick in ticks:
            tick['symbol']=symbol
        if not df:
            if not fields or fields == '':
                return ticks
            else:
                fields = load_to_list(fields)
                new_ticks = []
                for tick in ticks:
                    new_tick = {key: value for key, value in tick.items() if key in fields}
                    new_ticks.append(new_tick)
                return new_ticks


        if not ticks:
            return pd.DataFrame(columns=[fields])

        data = pd.DataFrame(ticks)
        if not fields or fields == '':
            return data
        else:
            fields = load_to_list(fields)
            return data[fields]

    else:

        frequency = _check_frequency(frequency)
        if not frequency:
            _error_call_back(ERR_INVALID_PARAMETER)
            return

        status, result = py_nfi_history_bars(symbol, frequency, start_time, end_time, fields, skip_suspended,
                                             fill_missing, 1)

        if not status == CSDK_OPERATE_SUCCESS:
           
            _error_call_back(status)
            return []

        bars = Bars()
        bars.ParseFromString(result)
        bars = [protobuf_to_dict(bars) for bars in bars.data]
        for bar in bars:
            bar['symbol'] = symbol
        if not df:
            if not fields or fields == '':
                return bars
            else:
                fields = load_to_list(fields)
                new_bars = []
                for bar in bars:
                    new_bar = {key: value for key, value in bar.items() if key in fields}
                    new_bars.append(new_bar)
                return new_bars


        if not bars:
            return pd.DataFrame(columns=[fields])

        data = pd.DataFrame(bars)
        if not fields or fields == '':
            return data
        else:
            fields = load_to_list(fields)
            return data[fields]


def history_n(symbol, frequency, count, end_time, fields='', skip_suspended=True, fill_missing='', df=False):
    if not symbol or not frequency or not count or not end_time:
        raise TypeError('history_n() : argument missing or incorrect argument type.')

    if len(symbol) <= 0:
        return []

    end_time = load_to_datetime_str(end_time)

    if frequency == 'tick':

        status, result = py_nfi_history_ticks_n(symbol, count, end_time, fields, skip_suspended,
                                                fill_missing, 1)

        if not status == CSDK_OPERATE_SUCCESS:
          
            _error_call_back(status)
            return []

        ticks = Ticks()
        ticks.ParseFromString(result)
        ticks = [protobuf_to_dict(tick) for tick in ticks.data]
        ticks = [_reform_tick_data(tick) for tick in ticks]
        for tick in ticks:
            tick['symbol']=symbol
        if not df:
            if not fields or fields == '':
                return ticks
            else:
                fields = load_to_list(fields)
                new_ticks = []
                for tick in ticks:
                    new_tick = {key: value for key, value in tick.items() if key in fields}
                    new_ticks.append(new_tick)
                return new_ticks


        if not ticks:
            return pd.DataFrame(columns=[fields])

        data = pd.DataFrame(ticks)
        if not fields or fields == '':
            return data
        else:
            fields = load_to_list(fields)
            return data[fields]

    else:
        frequency = _check_frequency(frequency)
        if not frequency:
            _error_call_back(ERR_INVALID_PARAMETER)
            return

        status, result = py_nfi_history_bars_n(symbol, frequency, count, end_time, fields, skip_suspended,
                                               fill_missing, 1)

        if not status == CSDK_OPERATE_SUCCESS:
           
            _error_call_back(status)
            return []

        bars = Bars()
        bars.ParseFromString(result)
        bars = [protobuf_to_dict(bars) for bars in bars.data]
        for bar in bars:
            bar['symbol']=symbol
        if not df:
            if not fields or fields == '':
                return bars
            else:
                fields = load_to_list(fields)
                new_bars = []
                for bar in bars:
                    new_bar = {key: value for key, value in bar.items() if key in fields}
                    new_bars.append(new_bar)
                return new_bars


        if not bars:
            return pd.DataFrame(columns=[fields])

        data = pd.DataFrame(bars)
        if not fields or fields == '':
            return data
        else:
            fields = load_to_list(fields)
            return data[fields]


def get_unfinished_orders_intraday(exchange=None):
    if not exchange:
        exchange = ''

    exchange = exchange.upper()

    status, result = py_nfi_get_unfinished_orders_intraday(exchange)

    if not status == CSDK_OPERATE_SUCCESS:
       
        _error_call_back(status)
        return []

    res = Orders()
    res.ParseFromString(result)

    orders = [protobuf_to_dict(res_order, including_default_value_fields=True) for res_order in res.data]

    if not exchange:
        return orders
    else:
        orders = [order for order in orders if exchange in order['symbol']]
        return orders


def get_instruments(symbol):
    if not symbol or len(symbol) <= 0:
        raise TypeError('get_instruments() : argument missing or incorrect argument type.')

    if isinstance(symbol, list):
        symbol = symbol[0]

    preq = SearchCoinPairsReq()
    preq.symbol.append(symbol)
    preq.subject = SecurityType_Physical
    req = preq.SerializeToString()

    status, result = py_nfi_get_instruments(req)

    if not status == CSDK_OPERATE_SUCCESS:
      
        _error_call_back(status)
        return []

    res = SearchCoinPairsRsp()
    res.ParseFromString(result)
    instrument = protobuf_to_dict(res)["symbolinfos"]

    return instrument


def order_cancel_all(exchange=""):
    exchange = exchange.upper()

    status = py_nfi_cancel_all_orders(exchange)

   
    return status


def get_exchange_rate(currencies, from_currency=None, start_time=None, end_time=None):
    """
    查询公允汇率，返回python list[dictionary]
    :param currencies:
    :param from_currency:
    :param start_time:
    :param end_time:
    :return:
    """
    if not currencies:
        raise TypeError('get_exchange_rate() : argument missing or incorrect argument type.')

    currencies_list = load_to_list(currencies)
    currencies = ','.join(currencies_list)

    if not from_currency:
        from_currency = 'USD'

    if not start_time:
        start_time = ''
    if not end_time:
        end_time = ''

    currencies = currencies.lower()
    from_currency = from_currency.lower()

    status, result = py_nfi_get_exchange_rate(currencies, from_currency, start_time, end_time)

    if not status == CSDK_OPERATE_SUCCESS:
      
        _error_call_back(status)
        return []

    # 反序列化。ExchangeRates是一个list类型，每个健对应的值是一个字典
    rates = ExchangeRateList()
    rates.ParseFromString(result)

    # 把返回的字典中的值取出来，放到一个字典列表中返回给用户
    rates = [protobuf_to_dict(currency, including_default_value_fields=True) for currency in rates.data]

    return rates


def get_depth(symbol, points=50):
    '''
    :param symbol:  trade symbol, like:BITFINEX.btcusd
    :param points:  levels of depth
    :return:    {'bids':[], 'asks':[]}
    :info: 用于获取交易深度
    '''
    if not symbol or len(symbol) <= 0:
        return

    if points > 50:
        points = 50

    status, result = py_nfi_get_market_depth(symbol)

    if not status == CSDK_OPERATE_SUCCESS:
        print('status: ', status)
        _error_call_back(status)
        return {}

    if len(result) <= 0:
        return {'points': 0, 'bids': [], 'asks': []}

    depth = DethpRateLists()
    depth.ParseFromString(result)

    depth = protobuf_to_dict(depth)

    bids = depth.data[0].bids
    asks = depth.data[0].asks

    if points < len(bids):
        bids = bids[0: points]
        asks = asks[0: points]

    bids = [dict_fields_filter(bid, ['price', 'volume']) for bid in bids]
    asks = [dict_fields_filter(ask, ['price', 'volume']) for ask in asks]

    bids.sort(key=takePriceFromDepth, reverse=True)     #买盘，降序
    asks.sort(key=takePriceFromDepth, reverse=False)    #卖盘，升序

    acc_vol = 0
    for dpt in asks:
        acc_vol += dpt['volume']
        dpt['accumulate'] = acc_vol

    acc_vol = 0
    for dpt in bids:
        acc_vol += dpt['volume']
        dpt['accumulate'] = acc_vol

    depth = {'points': len(bids), 'bids': bids, 'asks': asks}

    return depth


def alert_msg(message):
    if not message:
        return
    alert_req = AlertMeReq()
    alert_req.message = message
    req = alert_req.SerializeToString()

    status = py_nfi_alert_msg(req)
    if not status == CSDK_OPERATE_SUCCESS:
        _error_call_back(status)



def get_symbol_for_future(exchange, base, quote, period='', date=''):


    if not exchange or not base or not quote:
        raise TypeError('get_symbol_for_future() : argument missing or incorrect argument type.')
    if not period and not date:
        raise TypeError('get_symbol_for_future() : argument missing or incorrect argument type.')   
    preq = TargetSymbolReq()
    preq.market = exchange
    preq.front = base.lower()
    preq.rear = quote.lower()
    preq.contract_period = period.lower()
    preq.subject = SecurityType_Future
    preq.date=date
    req = preq.SerializeToString()

    status, result = py_nfi_get_symbol_for_future(req)
    if not status == CSDK_OPERATE_SUCCESS:
        _error_call_back(status)
        return None

    common_symbol_info = CommSymbolInfo()
    common_symbol_info.ParseFromString(result)
    '''
    common_symbol_info = [protobuf_to_dict(symbol_info, including_default_value_fields=True)
                          for symbol_info in common_symbol_info.data]
    symbol = {
        '{}'.format(symbol_info['symbol']): symbol_info for
        symbol_info in common_symbol_info}
    '''
    return common_symbol_info.data.symbole


def get_future_instruments(symbol, df=False):
    if not symbol:
        raise TypeError('get_future_instruments() : argument \'symbol\' must be provided.')
    print('api.trade.get_future_instruments: symbol=', symbol, ', ds=', df)
    preq = SearchCoinPairsReq()
    preq.symbol.append(symbol)
    preq.subject = SecurityType_Future
    req = preq.SerializeToString()

    status, result = py_nfi_get_instruments(req)

    if not status == CSDK_OPERATE_SUCCESS:
        _error_call_back(status)
        return

    res = SearchCoinPairsRsp()
    res.ParseFromString(result)

    instrument = protobuf_to_dict(res)["symbolinfos"]

    for symbol in instrument:
        symbol["symbol"] = symbol.pop("symbole")
        symbol["market"] = symbol.pop("marcket")
        symbol["symbol"] = symbol["market"] + "." + symbol["symbol"]
        symbol["settle_currency"] = symbol.pop("settlCurrency")
        symbol["period"] = symbol.pop("contract_period")
        if "settle_date" not in symbol.keys():
            symbol["settle_date"] = ""

    if df:
        instrument = pd.DataFrame(instrument)

    return instrument


def get_future_instruments_by_period(period, exchange=None, df=False):
    if not period:
        raise TypeError('get_future_instruments_by_period() : argument \'contract_period\' must be provided.')

    preq = SearchCoinPairsReq()
    if exchange:
        preq.exchange = exchange.upper()

    preq.subject = SecurityType_Future

    preq.contract_period = period.lower()
    # print('get_future_instruments_by_period preq = \n', preq)
    req = preq.SerializeToString()
    # print('get_future_instruments_by_period req = \n', req)

    status, result = py_nfi_get_instruments(req)
    # print('get_future_instruments_by_period status = ', status)
    if not status == CSDK_OPERATE_SUCCESS:
    
        _error_call_back(status)
        return

    res = SearchCoinPairsRsp()
    res.ParseFromString(result)
    # print('get_future_instruments_by_period: ', res)

    instrument = protobuf_to_dict(res)["symbolinfos"]
    for symbol in instrument:
        symbol["symbol"] = symbol.pop("symbole")
        symbol["market"] = symbol.pop("marcket")
        symbol["symbol"] = symbol["market"] + "." + symbol["symbol"]
        symbol["settle_currency"] = symbol.pop("settlCurrency")
        symbol["period"] = symbol.pop("contract_period")
        if "settle_date" not in symbol.keys():
            symbol["settle_date"] = ""
    if df:
        instrument = pd.DataFrame(instrument)

    return instrument


def get_future_instruments_by_base(base, period=None, df=False):
    if not base:
        raise TypeError('get_future_instruments_by_base() : argument \'base\' must be provided.')

    preq = SearchCoinPairsReq()
    if period:
        
        preq.contract_period = period.lower()

   
    preq.base = base.lower()
    preq.subject = SecurityType_Future
    # print('get_future_instruments_by_period preq = ', preq)
    req = preq.SerializeToString()

    status, result = py_nfi_get_instruments(req)

    if not status == CSDK_OPERATE_SUCCESS:
        
        _error_call_back(status)
        # print('get_future_instruments_by_base status = ', status)
        return

    res = SearchCoinPairsRsp()
    res.ParseFromString(result)

    instrument = protobuf_to_dict(res)["symbolinfos"]
    for symbol in instrument:
        symbol["symbol"] = symbol.pop("symbole")
        symbol["market"] = symbol.pop("marcket")
        symbol["symbol"] = symbol["market"] + "." + symbol["symbol"]
        symbol["settle_currency"] = symbol.pop("settlCurrency")
        symbol["period"] = symbol.pop("contract_period")
        if "settle_date" not in symbol.keys():
            symbol["settle_date"] = ""
    if df:
        instrument = pd.DataFrame(instrument)

    return instrument


def get_stats(base, df=False):
    if not base:
        raise TypeError('get_stats() : argument \'base\' must be provided.')

    preq = StatsReq()
   
    preq.Underlying_base = base.lower()

    req = preq.SerializeToString()

    status, result = py_nfi_get_target_stats(req)

    if not status == CSDK_OPERATE_SUCCESS:
     
        _error_call_back(status)
        return

    res = StatItems()
    res.ParseFromString(result)

    stats = protobuf_to_dict(res)["data"]

    if df:
        stats = pd.DataFrame(stats)

    return stats


def get_mark_price(symbol):
    if not symbol:
        raise TypeError('get_mark_price() : argument \'symbol\' must be provided.')

    status, price = py_nfi_get_mark_price(symbol)

    if not status == CSDK_OPERATE_SUCCESS:
       
        _error_call_back(status)
        return None

    return price


def get_settlements(symbol, count=100, end_time=None):
    if not symbol:
        raise TypeError('get_settlements() : argument \'symbol\' must be provided.')

    preq = GetSettlementReq()
    preq.data.symbol = symbol
    if count:
        preq.data.count = count
    if end_time:
        if isinstance(end_time, datetime):
            timestamp = end_time.timestamp()
            print(end_time, timestamp)
        preq.data.end_time.seconds = int(timestamp)

    req = preq.SerializeToString()

    status, result = py_nfi_get_settlements(req)

    if not status == CSDK_OPERATE_SUCCESS:
      
        _error_call_back(status)
        return

    res = SettlementItems()
    res.ParseFromString(result)

    stats = protobuf_to_dict(res)["data"]

    return stats


def get_funding_rate(symbol):
    if not symbol:
        raise TypeError('get_funding_rate() : argument \'symbol\' must be provided.')

    preq = SymbolReq()
    preq.symbol = symbol

    req = preq.SerializeToString()

    status, result = py_nfi_get_fundingRate(req)

    if not status == CSDK_OPERATE_SUCCESS:
       
        _error_call_back(status)
        return

    res = FundingRateResp()
    res.ParseFromString(result)

    rate = protobuf_to_dict(res)["data"]

    return rate


def get_margin_assets(exchange, currency=None):
    if not exchange:
        raise TypeError('get_margin_assets() : argument \'exchange\' must be provided.')

    preq = GetMarginAssetsReq()
    preq.exchange = exchange.upper()

    if currency:
        
        preq.currency = currency.lower()

    req = preq.SerializeToString()

    status, result = py_nfi_get_margin_assets(req)

    if not status == CSDK_OPERATE_SUCCESS:
        
        _error_call_back(status)
        return

    res = GetMarginAssetsResp()
    res.ParseFromString(result)

    asset = protobuf_to_dict(res)["data"]["data"]  # 要返回一个map,而不是嵌套的

    return asset


def set_mode(exchange, symbol, model):
    if not symbol or not model or not exchange:
        raise TypeError('set_mode() : argument missing or incorrect argument type.')

    if "." not in symbol:
     
        raise TypeError('set_mode() : incorrect symbol.')

    preq = SetModeReq()
    t_crc_list = symbol.split(".")
    preq.exchange = t_crc_list[0]
    preq.symbol = symbol
    preq.model = model
    preq.exchange = exchange.upper()

    req = preq.SerializeToString()

    status, result = py_nfi_set_margin_mode(req)

    if not status == CSDK_OPERATE_SUCCESS:

        _error_call_back(status)
        return

    res = StatResp()
    res.ParseFromString(result)

    rs = protobuf_to_dict(res)

    # print('sdk-set_mode: response = ', rs)

    rs = rs['data']

    if rs['ret'] != CSDK_OPERATE_SUCCESS:
        print('set_mode failed with reason: ', rs['reason'])

    return rs


def set_leverage(exchange, symbol, leverage):
    if not symbol or not exchange:
        raise TypeError('set_leverage() : argument missing or incorrect argument type.')

    if "." not in symbol:
        raise TypeError('set_leverage() : incorrect symbol.')

    preq = SetLeverageReq()
    t_crc_list = symbol.split(".")

    preq.exchange = t_crc_list[0]
    preq.symbol = symbol
    preq.leverage = leverage
    preq.exchange = exchange.upper()

    req = preq.SerializeToString()

    status, result = py_nfi_set_leverage(req)

    if not status == CSDK_OPERATE_SUCCESS:
        
        _error_call_back(status)
        return

    res = StatResp()
    res.ParseFromString(result)

    rs = protobuf_to_dict(res)

    # print('sdk-set_leverage: response = ', rs)
    rs = rs['data']

    if rs['ret'] != CSDK_OPERATE_SUCCESS:
        print('set_leverage failed with reason: ', rs['reason'])

    return rs


def add_margin(exchange, symbol, amount):
    if not symbol or not amount or not exchange:
        raise TypeError('add_margin() : argument missing or incorrect argument type.')

    if "." not in symbol:
        raise TypeError('set_leverage() : incorrect symbol.')

    preq = SetMarginReq()
    t_crc_list = symbol.split(".")
    preq.exchange = t_crc_list[0]
    preq.symbol = symbol
    preq.amount = amount
    preq.exchange = exchange.upper()

    req = preq.SerializeToString()

    status, result = py_nfi_add_margin(req)

    if not status == CSDK_OPERATE_SUCCESS:
        _error_call_back(status)
        return

    res = StatResp()
    res.ParseFromString(result)

    rs = protobuf_to_dict(res)

    print('sdk-add_margin: response = ', rs)

    rs = rs['data']

    if rs['ret'] != CSDK_OPERATE_SUCCESS:
        print('add_margin failed with reason: ', rs['reason'])

    return rs


def _created_time(ele):
    return ele['created_at']



# coding=utf-8
from __future__ import unicode_literals, print_function, absolute_import

import pandas as pd

from collections import deque

from nf.constant import SUB_TAG

from nf.model.storage import context


# 用来存储代码信息
class SubBarData(object):
    __instance = None

    def __new__(cls, *args, **kwd):
        if SubBarData.__instance is None:
            SubBarData.__instance = object.__new__(cls, *args, **kwd)
        return SubBarData.__instance

    def inside_init(self):
        self._data = {}

    def get_data(self, symbol, frequency, count, fields):
        sub_tag = SUB_TAG.format(symbol, frequency)
        if not self._data.get(sub_tag):
            return pd.DataFrame(columns=[fields])

        data = list(self._data[sub_tag].items)
        if not data:
            return pd.DataFrame(columns=[fields])

        data = pd.DataFrame(data)
        return data[fields].tail(count)

    def inside_get_data(self, symbols, frequency, bob):
        data = [self._inside_get_data(symbol, frequency, bob) for symbol in symbols]
        data = [info for info in data if info]
        return data

    def _inside_get_data(self, symbol, frequency, bob):
        sub_tag = SUB_TAG.format(symbol, frequency)
        if not self._data.get(sub_tag):
            return

        data = list(self._data[sub_tag].items)
        data.reverse()

        # 看看能否获取bar
        for bar in data:
            if bar['bob'] == bob:
                return bar

    def set_data(self, bar):
        if not isinstance(bar, dict):
            raise ValueError('set_data 应该传入的是dict类型')

        sub_tag = SUB_TAG.format(bar['symbol'], bar['frequency'])
        single_bar_storage = self._data.get(sub_tag)
        # 有些时候取消订阅了，也推送bar过来了， 这里做个容错处理
        if single_bar_storage is None:
            return

        # 回测时滑窗为0的时候， 尝试进行数据填充
        # if context.mode == MODE_BACKTEST and not single_bar_storage.items:
        #     single_bar_storage.data_init()

        # 如果滑窗缓存队列中没有数据， 尝试进行数据填充
        if not single_bar_storage.items:
            single_bar_storage.data_init()

        single_bar_storage.enqueue(bar)

    @property
    def largest_count(self):
        sub_list = [{'sub_name': sub.sub_tag, 'count': sub.count} for sub in context.inside_bar_subs]
        sub_list.sort(key=lambda obj: obj.get('count'), reverse=False)
        return {sub['sub_name']: sub['count'] for sub in sub_list}

    # 用户订阅数据后， contextdata应该初始化一个SubInfoQueue
    def sub_data(self, sub_tag, count):
        single_bar_storage = self._data.get(sub_tag)
        if not single_bar_storage:
            self._data[sub_tag] = BarInfoQueue(sub_tag, count)
            # 实时模式下, 数据填充放在订阅后
            # if context.mode == MODE_LIVE:
            #     self._data[sub_tag].data_init()
            return

        sub_info_queue = self._data.get(sub_tag)
        if sub_info_queue.count <= count:
            self._data[sub_tag] = BarInfoQueue(sub_tag, count)
            # 实时模式下, 数据填充放在订阅后
            # if context.mode == MODE_LIVE:
            #     self._data[sub_tag].data_init()
            return

    # 取消订阅直接清空就行了
    def unsub_data(self, sub_tag):
        if not self._data.get(sub_tag):
            return
        self._data.pop(sub_tag)



# 固定大小的先进先出队列
class BarInfoQueue(object):

    def __init__(self, sub_tag, count):
        self.sub_id = sub_tag
        self.count = count
        self.items = deque(maxlen=count)

    def enqueue(self, item):
        """进队列"""
        eobs = [i['eob'] for i in self.items]
        if item['eob'] not in eobs:
            self.items.append(item)

    def data_init(self):
        now = context.now.strftime("%Y-%m-%d %H:%M:%S")
        pre, symbol, frequency = self.sub_id.split(':')

        from nf.api import history_n

    #    if context.mode == MODE_BACKTEST:
    #        adjust_end_time = context.backtest_end_time
    #    else:
    #        adjust_end_time = ''

        datas = history_n(symbol, frequency, self.count, end_time=now)

    #    if frequency == '1d':
    #        for data in datas:
    #            data['eob'] = data['eob'].replace(hour=15, minute=15, second=1)

        [self.enqueue(data) for data in datas]

SubBarData().inside_init()



# coding=utf-8
from __future__ import unicode_literals, print_function, absolute_import

import datetime

from nf.enum import MODE_BACKTEST

from nf.utils import load_to_list, ObjectLikeDict, timestamp2datetime, check_frequency
from nf.csdk.c_sdk import nfi_now


class DefaultFileModule(object):
    def on_tick(self, tick):
        print('请初始化on_tick方法')

    def on_bar(self, bars):
        print('请初始化on_bar方法')

    def on_bod(self, bod):
        print('请初始化on_bod方法')

    def on_eod(self, eod):
        print('请初始化on_eod方法')


class Context(object):
    __instance = None

    def __new__(cls, *args, **kwd):
        if Context.__instance is None:
            Context.__instance = object.__new__(cls, *args, **kwd)
            cls.strategy_id = ''
            cls.inside_file_module = DefaultFileModule()
            cls.token = None
            cls.mode = None
            cls.temporary_now = None
            cls.backtest_start_time = None
            cls.backtest_end_time = None
            cls.adjust_mode = None
            cls.inside_schedules = {}

        return Context.__instance

    def inside_init(self):
        self._bar_subs = []
        self._bar_tasks = []
        self.inside_tick_subs = set()
        self.inside_accounts = {}

    def inside_unsubscribe_all(self):
        self.inside_init()
        from nf.model.storage_data import SubBarData
        SubBarData().inside_init()

    def __init__(self):
        self._bar_subs = []
        self._bar_tasks = []
        self.inside_tick_subs = set()
        self.inside_accounts = {}

    @property
    def now(self):
        '''
        返回一个datetime对象
        '''
        if self.temporary_now:
            return self.temporary_now

        now = nfi_now()
        # now == 0 说明是回测模式而且处于init装填 c sdk拿不到时间
        if now == 0:
            return datetime.datetime.strptime(context.backtest_start_time, "%Y-%m-%d %H:%M:%S")

        return timestamp2datetime(now)

    def inside_append_bar_sub(self, sub):
        if not sub.unique:
            return
        # 添加sub
        self._bar_subs.append(sub)
        # 添加任务
        self._append_bar_task(sub)
        # contextdata 添加相关类实例
        from nf.model.storage_data import SubBarData
        SubBarData().sub_data(sub.sub_tag, sub.count)


    def inside_remove_bar_sub(self, sub):
        # 移除sub
        self._bar_subs.remove(sub)
        # 取消任务
        self._remove_bar_task(sub)
        # contextdata 去掉相关类实例
        from nf.model.storage_data import SubBarData
        SubBarData().unsub_data(sub.sub_tag)

    @property
    def symbols(self):
        """
        bar 的symbols + tick 的symbols
        """
        return set([sub.symbol for sub in self.inside_bar_subs]).union(self.inside_tick_subs)

    # @property
    # def parameters(self):
    #     """
    #     动态参数
    #     """
    #     from nf.api.basic import get_parameters
    #     parameters = get_parameters()
    #     return {p['key']: p for p in parameters}

    @property
    def inside_bar_subs(self):
        return self._bar_subs

    @property
    def inside_bar_tasks(self):
        return self._bar_tasks

    @staticmethod
    def data(symbol, frequency, count=1, fields='symbol,bob,eob,open,high,low,close,volume'):
        fields = load_to_list(fields)
        frequency = check_frequency(frequency)

        from nf.model.storage_data import SubBarData
        return SubBarData().get_data(symbol, frequency, count, fields)

    def _append_bar_task(self, sub):
        # 如果不是wait_group的直接当成一个单独的任务就行
        if not sub.wait_group:
            self._bar_tasks.append(BarTask(symbols=[sub.symbol], frequency=sub.frequency, wait_group=False, wait_group_timeout=sub.wait_group_timeout))
            return

        # 如果没有该频率的task，构建一个， 有的话， 对这个task成员进行添加
        if sub.frequency not in [task.frequency for task in self.inside_bar_tasks if task.wait_group == True]:
            self._bar_tasks.append(BarTask(symbols=[sub.symbol], frequency=sub.frequency, wait_group=True, wait_group_timeout=sub.wait_group_timeout))

        else:
            [task.symbols.add(sub.symbol) for task in self.inside_bar_tasks if sub.frequency == task.frequency and task.wait_group == True]

    def _remove_bar_task(self, sub):
        # 如果不是wait_group的， 说明都是单独订阅的
        if not sub.wait_group:
            [self._bar_tasks.remove(task) for task in self._bar_tasks if task.symbols == [sub.symbol] and not task.wait_group]
            return

        # 如果是wait_group的， 要判断是否是单独的
        [self._bar_tasks.remove(task) for task in self._bar_tasks if task.symbols == [sub.symbol] and task.wait_group]
        # task.symbols != [sub.symbol] 说明还有一起存的
        [task.symbols.remove(sub.symbol) for task in self._bar_tasks if sub.symbol in task.symbols and task.wait_group]



# 提供给API的唯一上下文实例
context = Context()
context.inside_init()


# 任务中心 按 symbols， frequency 来分组
class BarTask(object):
    def __init__(self, symbols, frequency, wait_group, wait_group_timeout):
        self.waiting = False
        self.symbols = set(symbols)
        self.frequency = frequency
        self.wait_group = wait_group
        self.overtime = wait_group_timeout

        self.performed_eobs = set()
        self.performed_bobs = set()
        self.wait_perform_eob = None
        self.wait_perform_bob = None
        self.newest_eob = None
        self.newest_bob = None
        self.count_dict = {}

    def set_wait_perform_eob(self, bar):
        symbol = bar['symbol']
        eob = bar['eob']
        frequency = bar['frequency']

        if symbol not in self.symbols or not frequency == self.frequency:
            return

        self.newest_eob = eob
        self.count_dict[eob] = self.count_dict.get(eob, 0) + 1
        if not self.wait_perform_eob and eob not in self.performed_eobs:
            self.wait_perform_eob = eob

    def set_wait_perform_bob(self, bar):
        symbol = bar['symbol']
        bob = bar['bob']
        frequency = bar['frequency']

        if symbol not in self.symbols or not frequency == self.frequency:
            return

        self.newest_bob = bob
        self.count_dict[bob] = self.count_dict.get(bob, 0) + 1
        if not self.wait_perform_bob and bob not in self.performed_bobs:
            self.wait_perform_bob = bob

    # 每接受一个数据， 都会判断下是否满足条件， 如果满足条件， 将is_ready 改成true
    def state_analysis_mode_live(self):
        # 执行过的任务不在执行
        if context.now + datetime.timedelta(seconds=-1 * int(self.frequency[0:-1])) in self.performed_bobs:
            print('state_analysis_mode_live: ', context.now + datetime.timedelta(seconds=-1 * int(self.frequency[0:-1])))
            print('now time: ', timestamp2datetime(nfi_now()))
            return

        if not self.wait_perform_bob:
            return

        # 要判断一下是不是订阅的代码都到了啊。
        if self.count_dict[self.wait_perform_bob] >= len(self.symbols):
            return self.sign_waiting()

        # 如果超时时间到了， 任务就必须执行
        if (context.now + datetime.timedelta(seconds=-1 * int(self.frequency[0:-1])) - self.wait_perform_bob).total_seconds() >= self.overtime:
            self.sign_waiting()
            return

    def state_analysis_mode_backtest(self):
        # 执行过的任务不在执行
        if context.now + datetime.timedelta(seconds=-1 * int(self.frequency[0:-1])) in self.performed_bobs:
            return

        if not self.wait_perform_bob:
            return

        # 要判断一下是不是订阅的代码都到了啊。
        if self.count_dict[self.wait_perform_bob] >= len(self.symbols):
            return self.sign_waiting()

        # 如果超时时间到了， 任务就必须执行, 而且还得修改下context.now
        if (context.now + datetime.timedelta(seconds=-1 * int(self.frequency[0:-1])) - self.wait_perform_bob).total_seconds() >= 2:
            context.temporary_now = self.wait_perform_bob
            self.sign_waiting()
            return

    # 任务执行以后要重置一下
    def reset(self):
        self.waiting = False
        self.performed_bobs.add(self.wait_perform_bob)

        if self.newest_bob == self.wait_perform_bob:
            self.wait_perform_bob = None
        else:
            self.wait_perform_bob = self.newest_bob

        # 回测的话 temorory_now 调回去
        if context.mode == MODE_BACKTEST:
            context.temporary_now = None

    # 标记待执行任务和状态
    def sign_waiting(self):
        self.waiting = True

    # 获取等待返回的bar
    def get_perform_bars(self):
        from nf.model.storage_data import SubBarData
        bars = SubBarData().inside_get_data(self.symbols, self.frequency, self.wait_perform_bob)
        bars = [ObjectLikeDict(bar) for bar in bars]
        return bars



# coding=utf-8
from setuptools import setup
import nf.__version__ as version
import distutils.util

try:
    from wheel.bdist_wheel import bdist_wheel

    class _bdist_wheel(bdist_wheel):
        def get_tag(self):
            tag = bdist_wheel.get_tag(self)
            platform_tag = {
            'win32': 'win32',
            'win-amd64': 'win_amd64',
            'linux-x86_64': 'manylinux1_x86_64'}
            python_ver = tag[0]
            if python_ver == 'py27':
                python_ver = 'py2'
            if python_ver == 'py34':
                python_ver = 'py3'
            tag = (python_ver, tag[1], platform_tag[distutils.util.get_platform()])
            return tag

    cmdclass = {'bdist_wheel': _bdist_wheel}
except ImportError:
    cmdclass = {}


install_requires = [
    'typing==3.6.2',
    'arrow==0.10.0',
    'grpcio==1.4',
    'Cython==0.26.1',
    'protobuf==3.4.0',
]

dependency_links = [
    'https://download.lfd.uci.edu/pythonlibs/j1ulh5xc/TA_Lib-0.4.17-cp36-cp36m-win_amd64.whl',
]


setup(
    name=version.__title__,
    version=version.__version__,
    description=version.__description__,
    url=version.__url__,
    author=version.__author__,
    author_email=version.__author_email__,
    license=version.__license__,
    include_package_data=True,

    # 要打的包的所有文件夹
    packages=['nf', 'nf.model', 'nf.api', 'nf.pb', 'nf.csdk', 'nf.csdk.lib',
              'github.com', 'github.com.gogo', 'github.com.gogo.protobuf',
              'github.com.gogo.protobuf.gogoproto', 'google.api',
              'nf.pb.auth', 'nf.pb.core', 'nf.pb.data',  'nf.pb.strategy',
              'nf.pb.trade',  'nf.pb.tradegw',

              'nf.pb.auth.api', 'nf.pb.auth.api.auth',
              'nf.pb.core.api', 'nf.pb.core.api.report',
              'nf.pb.data.api',
              'nf.pb.strategy.api', 'nf.pb.strategy.api.strategy',
              'nf.pb.strategy.strategy_params.strategy',
              'nf.pb.trade.api', 'nf.pb.trade.api.trade',
              'nf.pb.tradegw.api', 'nf.pb.tradegw.api.tradegw',
              'nf.pb.gogoproto',
              'third_lib'

              ],
    zip_safe=False,
    # 依赖库
    install_requires=install_requires,
    dependency_links=dependency_links,
    extras_require={
        'pandas': ['pandas'],
    },
    # 包对应文件
    package_dir={
        'nf': './nf',
        'nf.api': './nf/api',
        'github': './github',
        'google': './google',
        'nf.csdk:': './nf/csdk',
        'nf.csdk.lib:': './nf/csdk',
        'nf.model:': './nf/model',
        'nf.pb': './nf/pb',
        'nf.pb.auth': './nf/pb/auth',
        'nf.pb.core': './nf/pb/core',
        'nf.pb.data': './nf/pb/data',
        'nf.pb.strategy': './nf/pb/strategy',
        'nf.pb.trade': './nf/pb/trade',
        'nf.pb.tradegw': './nf/pb/tradegw',

        'nf.pb.auth.api': './nf/pb/auth/api',
        'nf.pb.auth.api.auth': './nf/pb/auth/api/auth',

        'nf.pb.gogoproto': './nf/pb/gogoproto',

        'nf.pb.core.api': './nf/pb/core/api',
        'nf.pb.core.api.report': './nf/pb/core/api/report',

        'nf.pb.data.api': './nf/pb/data/api',
        'nf.pb.strategy.api': './nf/pb/strategy/api',
        'nf.pb.strategy.api.strategy': './nf/pb/strategy/api/strategy',
        'nf.pb.strategy.strategy_params.strategy': './nf/pb/strategy/strategy_params/strategy',

        'nf.pb.trade.api': './nf/pb/trade/api',
        'nf.pb.trade.api.trade': './nf/pb/trade/api/trade',

        'nf.pb.tradegw.api': './nf/pb/tradegw/api',
        'nf.pb.tradegw.api.tradegw': './nf/pb/tradegw/api/tradegw',

        'github.com': './github/com',
        'github.com.gogo': './github/com/gogo',
        'github.com.gogo/protobuf': './github/com/gogo/protobuf',
        'github.com.gogo/protobuf/gogoproto': './github/com/gogo/protobuf/gogoproto',
        'third_lib': './third_lib'
    },
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ],
    # 命名空间， 要么有 __init__.py  要么就要一个写一遍
    namespace_packages=[
        'nf',
        'google',
        'github',
    ],
    python_requires='>=2.7, !=3.0.*, !=3.1.*, !=3.2.*, !=3.3.*, !=3.4.*, !=3.5.*, <4',
    cmdclass=cmdclass,
    package_data={
        'nf.csdk': ['*.so'],
        'third_lib': ['third_lib/TA_Lib-0.4.17-cp36-cp36m-win_amd64.whl'],
    },
    # data_files=[('%AppData%/third_lib', ['third_lib/TA_Lib-0.4.17-cp36-cp36m-win_amd64.whl'])],
    # scripts=['install_talib.bat'],



)




# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: strategy/strategy_params/strategy.params.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='strategy/strategy_params/strategy.params.proto',
  package='strategy.params',
  syntax='proto3',
  serialized_pb=_b('\n.strategy/strategy_params/strategy.params.proto\x12\x0fstrategy.params\"\xcc\x04\n\x14StrategyDuidaoParams\x12\x13\n\x0blogFilePath\x18\x01 \x01(\t\x12\x10\n\x08\x65xchange\x18\x04 \x01(\t\x12\r\n\x05\x66ront\x18\x05 \x01(\t\x12\x0c\n\x04rear\x18\x06 \x01(\t\x12\x0e\n\x06symbol\x18\x07 \x01(\t\x12\x12\n\nsymbolMark\x18\x08 \x01(\t\x12\x18\n\x10\x63\x61lmDownTimeStep\x18\t \x01(\x03\x12\x18\n\x10\x63\x61lmDownTimeStop\x18\n \x01(\x03\x12\x0e\n\x06volSum\x18\x0b \x01(\x01\x12\x0f\n\x07timeSum\x18\x0c \x01(\x03\x12\x10\n\x08stepTime\x18\r \x01(\x03\x12\x0e\n\x06\x62idNum\x18\x0e \x01(\x03\x12\x0e\n\x06\x61skNum\x18\x0f \x01(\x03\x12\x14\n\x0csingleVolMin\x18\x10 \x01(\x01\x12\x14\n\x0csingleVolMax\x18\x11 \x01(\x01\x12\x11\n\tinitPrice\x18\x12 \x01(\x01\x12\x0f\n\x07testVol\x18\x13 \x01(\x01\x12\x14\n\x0cinvVolumnNum\x18\x14 \x01(\x01\x12\x1f\n\x17singleDoneBuyRateWaring\x18\x15 \x01(\x01\x12 \n\x18SingleDoneSellRateWaring\x18\x16 \x01(\x01\x12\x1c\n\x14\x43\x61lDoneBuyRateWaring\x18\x17 \x01(\x01\x12\x1d\n\x15\x43\x61lDoneSellRateWaring\x18\x18 \x01(\x01\x12\x15\n\rsymbolDecimal\x18\x19 \x01(\x01\x12\x18\n\x10symbolDecimalNum\x18\x1a \x01(\x03\x12\x17\n\x0f\x61\x62sPriceCelling\x18\x1b \x01(\x01\x12\x15\n\rabsPriceFloor\x18\x1c \x01(\x01\"\xe8\x02\n\x13StrategyBudanParams\x12\x13\n\x0blogFilePath\x18\x01 \x01(\t\x12\x10\n\x08\x65xchange\x18\x04 \x01(\t\x12\r\n\x05\x66ront\x18\x05 \x01(\t\x12\x0c\n\x04rear\x18\x06 \x01(\t\x12\x0e\n\x06symbol\x18\x07 \x01(\t\x12\x11\n\tbuyOrSell\x18\x08 \x01(\x03\x12\x14\n\x0cshiftOrPrice\x18\t \x01(\x03\x12\x0e\n\x06\x62order\x18\n \x01(\x01\x12\x0f\n\x07lineNum\x18\x0b \x01(\x03\x12\x0e\n\x06volSum\x18\x0c \x01(\x01\x12\x0e\n\x06volMin\x18\r \x01(\x01\x12\x11\n\tinitPrice\x18\x0e \x01(\x01\x12\r\n\x05state\x18\x0f \x01(\x03\x12\x0e\n\x06tdHigh\x18\x10 \x01(\x03\x12\r\n\x05tdLow\x18\x11 \x01(\x03\x12\x12\n\nwaringRate\x18\x12 \x01(\x01\x12\x11\n\tstopOrNot\x18\x13 \x01(\x03\x12\x11\n\tunitPrice\x18\x14 \x01(\x01\x12\x18\n\x10symbolDecimalNum\x18\x15 \x01(\x03\"\xfa\x02\n\x13StrategyLapanParams\x12\x13\n\x0blogFilePath\x18\x01 \x01(\t\x12\x10\n\x08\x65xchange\x18\x04 \x01(\t\x12\r\n\x05\x66ront\x18\x05 \x01(\t\x12\x0c\n\x04rear\x18\x06 \x01(\t\x12\x0e\n\x06symbol\x18\x07 \x01(\t\x12\x0c\n\x04mode\x18\x08 \x01(\x03\x12\x15\n\rmeanTimeIntvl\x18\t \x01(\x03\x12\x16\n\x0etotalTimeLimit\x18\n \x01(\x03\x12\x14\n\x0cmeanOrderVol\x18\x0b \x01(\x01\x12\x17\n\x0fpriceChangeRate\x18\x0c \x01(\x01\x12\x16\n\x0emeanPriceLimit\x18\r \x01(\x01\x12\x15\n\rtotalVolLimit\x18\x0e \x01(\x01\x12\x13\n\x0bpriceTarget\x18\x0f \x01(\x01\x12\x15\n\rprecautionVol\x18\x10 \x01(\x01\x12\x17\n\x0fprecautionPrice\x18\x11 \x01(\x01\x12\x15\n\rsymbolDecimal\x18\x12 \x01(\x01\x12\x18\n\x10symbolDecimalNum\x18\x13 \x01(\x03\x62\x06proto3')
)




_STRATEGYDUIDAOPARAMS = _descriptor.Descriptor(
  name='StrategyDuidaoParams',
  full_name='strategy.params.StrategyDuidaoParams',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='logFilePath', full_name='strategy.params.StrategyDuidaoParams.logFilePath', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='exchange', full_name='strategy.params.StrategyDuidaoParams.exchange', index=1,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='front', full_name='strategy.params.StrategyDuidaoParams.front', index=2,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='rear', full_name='strategy.params.StrategyDuidaoParams.rear', index=3,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='symbol', full_name='strategy.params.StrategyDuidaoParams.symbol', index=4,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='symbolMark', full_name='strategy.params.StrategyDuidaoParams.symbolMark', index=5,
      number=8, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='calmDownTimeStep', full_name='strategy.params.StrategyDuidaoParams.calmDownTimeStep', index=6,
      number=9, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='calmDownTimeStop', full_name='strategy.params.StrategyDuidaoParams.calmDownTimeStop', index=7,
      number=10, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='volSum', full_name='strategy.params.StrategyDuidaoParams.volSum', index=8,
      number=11, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='timeSum', full_name='strategy.params.StrategyDuidaoParams.timeSum', index=9,
      number=12, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='stepTime', full_name='strategy.params.StrategyDuidaoParams.stepTime', index=10,
      number=13, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='bidNum', full_name='strategy.params.StrategyDuidaoParams.bidNum', index=11,
      number=14, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='askNum', full_name='strategy.params.StrategyDuidaoParams.askNum', index=12,
      number=15, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='singleVolMin', full_name='strategy.params.StrategyDuidaoParams.singleVolMin', index=13,
      number=16, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='singleVolMax', full_name='strategy.params.StrategyDuidaoParams.singleVolMax', index=14,
      number=17, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='initPrice', full_name='strategy.params.StrategyDuidaoParams.initPrice', index=15,
      number=18, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='testVol', full_name='strategy.params.StrategyDuidaoParams.testVol', index=16,
      number=19, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='invVolumnNum', full_name='strategy.params.StrategyDuidaoParams.invVolumnNum', index=17,
      number=20, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='singleDoneBuyRateWaring', full_name='strategy.params.StrategyDuidaoParams.singleDoneBuyRateWaring', index=18,
      number=21, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='SingleDoneSellRateWaring', full_name='strategy.params.StrategyDuidaoParams.SingleDoneSellRateWaring', index=19,
      number=22, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='CalDoneBuyRateWaring', full_name='strategy.params.StrategyDuidaoParams.CalDoneBuyRateWaring', index=20,
      number=23, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='CalDoneSellRateWaring', full_name='strategy.params.StrategyDuidaoParams.CalDoneSellRateWaring', index=21,
      number=24, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='symbolDecimal', full_name='strategy.params.StrategyDuidaoParams.symbolDecimal', index=22,
      number=25, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='symbolDecimalNum', full_name='strategy.params.StrategyDuidaoParams.symbolDecimalNum', index=23,
      number=26, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='absPriceCelling', full_name='strategy.params.StrategyDuidaoParams.absPriceCelling', index=24,
      number=27, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='absPriceFloor', full_name='strategy.params.StrategyDuidaoParams.absPriceFloor', index=25,
      number=28, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=68,
  serialized_end=656,
)


_STRATEGYBUDANPARAMS = _descriptor.Descriptor(
  name='StrategyBudanParams',
  full_name='strategy.params.StrategyBudanParams',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='logFilePath', full_name='strategy.params.StrategyBudanParams.logFilePath', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='exchange', full_name='strategy.params.StrategyBudanParams.exchange', index=1,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='front', full_name='strategy.params.StrategyBudanParams.front', index=2,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='rear', full_name='strategy.params.StrategyBudanParams.rear', index=3,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='symbol', full_name='strategy.params.StrategyBudanParams.symbol', index=4,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='buyOrSell', full_name='strategy.params.StrategyBudanParams.buyOrSell', index=5,
      number=8, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='shiftOrPrice', full_name='strategy.params.StrategyBudanParams.shiftOrPrice', index=6,
      number=9, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='border', full_name='strategy.params.StrategyBudanParams.border', index=7,
      number=10, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='lineNum', full_name='strategy.params.StrategyBudanParams.lineNum', index=8,
      number=11, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='volSum', full_name='strategy.params.StrategyBudanParams.volSum', index=9,
      number=12, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='volMin', full_name='strategy.params.StrategyBudanParams.volMin', index=10,
      number=13, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='initPrice', full_name='strategy.params.StrategyBudanParams.initPrice', index=11,
      number=14, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='state', full_name='strategy.params.StrategyBudanParams.state', index=12,
      number=15, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='tdHigh', full_name='strategy.params.StrategyBudanParams.tdHigh', index=13,
      number=16, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='tdLow', full_name='strategy.params.StrategyBudanParams.tdLow', index=14,
      number=17, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='waringRate', full_name='strategy.params.StrategyBudanParams.waringRate', index=15,
      number=18, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='stopOrNot', full_name='strategy.params.StrategyBudanParams.stopOrNot', index=16,
      number=19, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='unitPrice', full_name='strategy.params.StrategyBudanParams.unitPrice', index=17,
      number=20, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='symbolDecimalNum', full_name='strategy.params.StrategyBudanParams.symbolDecimalNum', index=18,
      number=21, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=659,
  serialized_end=1019,
)


_STRATEGYLAPANPARAMS = _descriptor.Descriptor(
  name='StrategyLapanParams',
  full_name='strategy.params.StrategyLapanParams',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='logFilePath', full_name='strategy.params.StrategyLapanParams.logFilePath', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='exchange', full_name='strategy.params.StrategyLapanParams.exchange', index=1,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='front', full_name='strategy.params.StrategyLapanParams.front', index=2,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='rear', full_name='strategy.params.StrategyLapanParams.rear', index=3,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='symbol', full_name='strategy.params.StrategyLapanParams.symbol', index=4,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='mode', full_name='strategy.params.StrategyLapanParams.mode', index=5,
      number=8, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='meanTimeIntvl', full_name='strategy.params.StrategyLapanParams.meanTimeIntvl', index=6,
      number=9, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='totalTimeLimit', full_name='strategy.params.StrategyLapanParams.totalTimeLimit', index=7,
      number=10, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='meanOrderVol', full_name='strategy.params.StrategyLapanParams.meanOrderVol', index=8,
      number=11, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='priceChangeRate', full_name='strategy.params.StrategyLapanParams.priceChangeRate', index=9,
      number=12, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='meanPriceLimit', full_name='strategy.params.StrategyLapanParams.meanPriceLimit', index=10,
      number=13, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='totalVolLimit', full_name='strategy.params.StrategyLapanParams.totalVolLimit', index=11,
      number=14, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='priceTarget', full_name='strategy.params.StrategyLapanParams.priceTarget', index=12,
      number=15, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='precautionVol', full_name='strategy.params.StrategyLapanParams.precautionVol', index=13,
      number=16, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='precautionPrice', full_name='strategy.params.StrategyLapanParams.precautionPrice', index=14,
      number=17, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='symbolDecimal', full_name='strategy.params.StrategyLapanParams.symbolDecimal', index=15,
      number=18, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='symbolDecimalNum', full_name='strategy.params.StrategyLapanParams.symbolDecimalNum', index=16,
      number=19, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1022,
  serialized_end=1400,
)

DESCRIPTOR.message_types_by_name['StrategyDuidaoParams'] = _STRATEGYDUIDAOPARAMS
DESCRIPTOR.message_types_by_name['StrategyBudanParams'] = _STRATEGYBUDANPARAMS
DESCRIPTOR.message_types_by_name['StrategyLapanParams'] = _STRATEGYLAPANPARAMS
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

StrategyDuidaoParams = _reflection.GeneratedProtocolMessageType('StrategyDuidaoParams', (_message.Message,), dict(
  DESCRIPTOR = _STRATEGYDUIDAOPARAMS,
  __module__ = 'strategy.strategy_params.strategy.params_pb2'
  # @@protoc_insertion_point(class_scope:strategy.params.StrategyDuidaoParams)
  ))
_sym_db.RegisterMessage(StrategyDuidaoParams)

StrategyBudanParams = _reflection.GeneratedProtocolMessageType('StrategyBudanParams', (_message.Message,), dict(
  DESCRIPTOR = _STRATEGYBUDANPARAMS,
  __module__ = 'strategy.strategy_params.strategy.params_pb2'
  # @@protoc_insertion_point(class_scope:strategy.params.StrategyBudanParams)
  ))
_sym_db.RegisterMessage(StrategyBudanParams)

StrategyLapanParams = _reflection.GeneratedProtocolMessageType('StrategyLapanParams', (_message.Message,), dict(
  DESCRIPTOR = _STRATEGYLAPANPARAMS,
  __module__ = 'strategy.strategy_params.strategy.params_pb2'
  # @@protoc_insertion_point(class_scope:strategy.params.StrategyLapanParams)
  ))
_sym_db.RegisterMessage(StrategyLapanParams)


# @@protoc_insertion_point(module_scope)



# coding=utf-8
from __future__ import unicode_literals, print_function, absolute_import

from nf.constant import SUB_ID, SUB_TAG
from nf.model.storage import context


class SubDetail(object):
    def __init__(self, symbol, frequency, count, wait_group, wait_group_timeout, unsubscribe_previous):
        #print("=========>",symbol)
        self.symbol = symbol
        self.frequency = frequency
        self.count = count
        self.wait_group = wait_group
        self.wait_group_timeout = wait_group_timeout
        self.unsubscribe_previous = unsubscribe_previous
        self.id = SUB_ID.format(self.symbol, self.frequency, self.count)

    @property
    def unique(self):
        # 看看是否订阅过
        return self.id not in [sub.id for sub in context.inside_bar_subs]

    def __repr__(self):
        return self.id

    @property
    def sub_tag(self):
        # 取消订阅标签& 查找订阅最大值时用
        return SUB_TAG.format(self.symbol, self.frequency)



# -*- coding:utf-8 -*-
from __future__ import unicode_literals, print_function, absolute_import

from datetime import datetime, timedelta

from nf.utils import DictLikeObject

"""
主要代码来自protobuf_to_dict， 对其进行了部分修改
"""
import six

from google.protobuf.message import Message
from google.protobuf.descriptor import FieldDescriptor
from google.protobuf.timestamp_pb2 import Timestamp

__all__ = ["protobuf_to_dict", "TYPE_CALLABLE_MAP", "dict_to_protobuf",
           "REVERSE_TYPE_CALLABLE_MAP"]


EXTENSION_CONTAINER = '___X'


TYPE_CALLABLE_MAP = {
    FieldDescriptor.TYPE_DOUBLE: float,
    FieldDescriptor.TYPE_FLOAT: float,
    FieldDescriptor.TYPE_INT32: int,
    FieldDescriptor.TYPE_INT64: int if six.PY3 else six.integer_types[1],
    FieldDescriptor.TYPE_UINT32: int,
    FieldDescriptor.TYPE_UINT64: int if six.PY3 else six.integer_types[1],
    FieldDescriptor.TYPE_SINT32: int,
    FieldDescriptor.TYPE_SINT64: int if six.PY3 else six.integer_types[1],
    FieldDescriptor.TYPE_FIXED32: int,
    FieldDescriptor.TYPE_FIXED64: int if six.PY3 else six.integer_types[1],
    FieldDescriptor.TYPE_SFIXED32: int,
    FieldDescriptor.TYPE_SFIXED64: int if six.PY3 else six.integer_types[1],
    FieldDescriptor.TYPE_BOOL: bool,
    FieldDescriptor.TYPE_STRING: six.text_type,
    FieldDescriptor.TYPE_BYTES: six.binary_type,
    FieldDescriptor.TYPE_ENUM: int,
}


def repeated(type_callable):
    return lambda value_list: [type_callable(value) for value in value_list]


def enum_label_name(field, value):
    return field.enum_type.values_by_number[int(value)].name


def _is_map_entry(field):
    return (field.type == FieldDescriptor.TYPE_MESSAGE and
            field.message_type.has_options and
            field.message_type.GetOptions().map_entry)


def protobuf_to_dict(pb, type_callable_map=TYPE_CALLABLE_MAP, use_enum_labels=False,
                     including_default_value_fields=True, is_utc_time=True):
    result_dict = DictLikeObject()
    extensions = {}

    for field, value in pb.ListFields():

        if isinstance(value, Timestamp):
            # 这个方法是对value.ToDatetime()的改造， 防止1969年报错
            _NANOS_PER_SECOND = 1000000000
            deltasec = value.seconds + value.nanos / float(_NANOS_PER_SECOND)
            result_dict[field.name] = (datetime(1970, 1, 1) + timedelta(seconds=deltasec))

            if not is_utc_time:
                result_dict[field.name] = result_dict[field.name] - timedelta(hours=8)

            continue

        if field.message_type and field.message_type.has_options and field.message_type.GetOptions().map_entry:
            result_dict[field.name] = dict()
            value_field = field.message_type.fields_by_name['value']
            type_callable = _get_field_value_adaptor(
                pb, value_field, type_callable_map,
                use_enum_labels, including_default_value_fields)
            for k, v in value.items():
                result_dict[field.name][k] = type_callable(v)
            continue
        type_callable = _get_field_value_adaptor(pb, field, type_callable_map,
                                                 use_enum_labels, including_default_value_fields)
        if field.label == FieldDescriptor.LABEL_REPEATED:
            type_callable = repeated(type_callable)

        if field.is_extension:
            extensions[str(field.number)] = type_callable(value)
            continue

        result_dict[field.name] = type_callable(value)

    # Serialize default value if including_default_value_fields is True.
    if including_default_value_fields:
        for field in pb.DESCRIPTOR.fields:
            # Singular message fields and oneof fields will not be affected.
            if ((
                    field.label != FieldDescriptor.LABEL_REPEATED and
                    field.cpp_type == FieldDescriptor.CPPTYPE_MESSAGE) or
                    field.containing_oneof):
                continue
            if field.name in result_dict:
                # Skip the field which has been serailized already.
                continue
            if _is_map_entry(field):
                result_dict[field.name] = {}
            else:
                result_dict[field.name] = field.default_value

    if extensions:
        result_dict[EXTENSION_CONTAINER] = extensions
    return result_dict


def _get_field_value_adaptor(pb, field, type_callable_map=TYPE_CALLABLE_MAP, use_enum_labels=False,
                             including_default_value_fields=False):
    if field.type == FieldDescriptor.TYPE_MESSAGE:
        # recursively encode protobuf sub-message
        return lambda pb: protobuf_to_dict(
            pb, type_callable_map=type_callable_map,
            use_enum_labels=use_enum_labels,
            including_default_value_fields=including_default_value_fields,
        )

    if use_enum_labels and field.type == FieldDescriptor.TYPE_ENUM:
        return lambda value: enum_label_name(field, value)

    if field.type in type_callable_map:
        return type_callable_map[field.type]

    raise TypeError("Field %s.%s has unrecognised type id %d" % (
        pb.__class__.__name__, field.name, field.type))


REVERSE_TYPE_CALLABLE_MAP = {
}


def dict_to_protobuf(pb_klass_or_instance, values, type_callable_map=REVERSE_TYPE_CALLABLE_MAP, strict=True, ignore_none=False):
    """Populates a protobuf model from a dictionary.

    :param pb_klass_or_instance: a protobuf message class, or an protobuf instance
    :type pb_klass_or_instance: a type or instance of a subclass of google.protobuf.message.Message
    :param dict values: a dictionary of values. Repeated and nested values are
       fully supported.
    :param dict type_callable_map: a mapping of protobuf types to callables for setting
       values on the target instance.
    :param bool strict: complain if keys in the map are not fields on the message.
    :param bool strict: ignore None-values of fields, treat them as empty field
    """
    if isinstance(pb_klass_or_instance, Message):
        instance = pb_klass_or_instance
    else:
        instance = pb_klass_or_instance()
    return _dict_to_protobuf(instance, values, type_callable_map, strict, ignore_none)


def _get_field_mapping(pb, dict_value, strict):
    field_mapping = []
    for key, value in dict_value.items():
        if key == EXTENSION_CONTAINER:
            continue
        if key not in pb.DESCRIPTOR.fields_by_name:
            if strict:
                raise KeyError("%s does not have a field called %s" % (pb, key))
            continue
        field_mapping.append((pb.DESCRIPTOR.fields_by_name[key], value, getattr(pb, key, None)))

    for ext_num, ext_val in dict_value.get(EXTENSION_CONTAINER, {}).items():
        try:
            ext_num = int(ext_num)
        except ValueError:
            raise ValueError("Extension keys must be integers.")
        if ext_num not in pb._extensions_by_number:
            if strict:
                raise KeyError("%s does not have a extension with number %s. Perhaps you forgot to import it?" % (pb, key))
            continue
        ext_field = pb._extensions_by_number[ext_num]
        pb_val = None
        pb_val = pb.Extensions[ext_field]
        field_mapping.append((ext_field, ext_val, pb_val))

    return field_mapping


def _dict_to_protobuf(pb, value, type_callable_map, strict, ignore_none):
    fields = _get_field_mapping(pb, value, strict)

    for field, input_value, pb_value in fields:
        if ignore_none and input_value is None:
            continue
        if field.label == FieldDescriptor.LABEL_REPEATED:
            if field.message_type and field.message_type.has_options and field.message_type.GetOptions().map_entry:
                value_field = field.message_type.fields_by_name['value']
                for key, value in input_value.items():
                    if value_field.cpp_type == FieldDescriptor.CPPTYPE_MESSAGE:
                        _dict_to_protobuf(getattr(pb, field.name)[key], value, type_callable_map, strict, ignore_none)
                    else:
                        getattr(pb, field.name)[key] = value
                continue
            for item in input_value:
                if field.type == FieldDescriptor.TYPE_MESSAGE:
                    m = pb_value.add()
                    _dict_to_protobuf(m, item, type_callable_map, strict, ignore_none)
                elif field.type == FieldDescriptor.TYPE_ENUM and isinstance(item, six.string_types):
                    pb_value.append(_string_to_enum(field, item))
                else:
                    pb_value.append(item)
            continue
        if field.type == FieldDescriptor.TYPE_MESSAGE:
            _dict_to_protobuf(pb_value, input_value, type_callable_map, strict, ignore_none)
            continue

        if field.type in type_callable_map:
            input_value = type_callable_map[field.type](input_value)

        if field.is_extension:
            pb.Extensions[field] = input_value
            continue

        if field.type == FieldDescriptor.TYPE_ENUM and isinstance(input_value, six.string_types):
            input_value = _string_to_enum(field, input_value)

        setattr(pb, field.name, input_value)

    return pb


def _string_to_enum(field, input_value):
    enum_dict = field.enum_type.values_by_name
    try:
        input_value = enum_dict[input_value].number
    except KeyError:
        raise KeyError("`%s` is not a valid value for field `%s`" % (input_value, field.name))
    return input_value
